import collections
# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv, fields
from tools.translate import _

_MIN_DATETIME='1970-01-01 00:00:00'
_MAX_DATETIME='2099-12-31 23:59:59'

class wizard_complete_state(osv.osv_memory):
    _name = 'contact.wizard_complete_state'

    _columns = {
        'contact_id' : fields.many2one('res.partner.contact','Contact',readonly=True,),
        'domain_anno_types' : fields.text('Annotation Type Domain',readonly=True,),
        'anno_type_id' : fields.many2one('contact.type_annotation','Annotation type',domain="[('id','in',map(int,domain_anno_types.split(',')) )]"),
        'datetime_from' : fields.datetime('From',),
        'datetime_to' : fields.datetime('To',),
        'permanent' : fields.boolean('Permanent'),
        'state' : fields.selection([('draft','Draft'),('executed','Executed')], string='State',),
    }
    
    def default_get(self, cr, uid, fields, context = None):
        ret = {'permanent': True,}

        if not context: context = {}
        ids = context.get('active_ids',[])
        if not ids: return ret
        
        dicci = collections.defaultdict(set)
        state_type_ids_set = set()
        for x in self.pool.get('contact.state_range2').browse(cr, uid, ids):
            if 'contact_id' in ret and ret['contact_id'] != x.contact_id.id:
                pass # Error ?
            ret['contact_id'] = x.contact_id.id
            state_type_ids_set.add(x.lstate.id)
            if 'datetime_from' not in ret or ret['datetime_from']>x.datetime_from:
                ret['datetime_from'] = x.datetime_from
            if 'datetime_to' not in ret or ret['datetime_to']<x.datetime_to:
                ret['datetime_to'] = x.datetime_to        
        if not fields or 'anno_type_id' in fields or 'domain_anno_types' in fields:
            state_type_ids = list(state_type_ids_set)
            anno_type_ids2 = self.pool.get('contact.type_annotation').get_type_annotations(cr, uid, contact_ids=[ret['contact_id']], task='create')
            anno_type_ids = self.pool.get('contact.anno_type.state').search(cr, uid, [('state_id','in',list(state_type_ids)),('op','=',True),('anno_type','in',anno_type_ids2)])
            map(lambda x: dicci[x.anno_type.id].add(x.state_id.id), self.pool.get('contact.anno_type.state').browse(cr, uid, anno_type_ids))
            domain_anno_type_ids = [anno_type_id for (anno_type_id,states_set) in filter(lambda (anno_type_id,states_set): len(states_set) == len(state_type_ids), dicci.items())]
            ret['anno_type_id'] = anno_type_ids and anno_type_ids[0] or False
            ret['domain_anno_types'] = ','.join(map(str,domain_anno_type_ids))
        if fields:
            ret=dict((key,value) for (key,value) in ret.items() if key in fields)
        return ret
        
    def act_create(self, cr, uid, ids, context = None):
        for item in self.browse(cr, uid, ids):
            if self.pool.get('contact.annotation').create(cr, uid, {'contact_id': item.contact_id.id,
                                                                 'anno_type': item.anno_type_id.id,
                                                                 'valid_from': item.datetime_from,
                                                                 'valid_to': item.datetime_to,
                                                                 'permanent': item.permanent,}):
                self.write(cr, uid, [item.id], {'state': 'executed',})
        return True
    
    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}
    
wizard_complete_state()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
