# -*- encoding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school modules
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import pooler
from report.interface import report_rml
from report.interface import toxml
from datetime import datetime, timedelta
from lxml import etree

#FIXME: we should use toxml

class report_custom(report_rml):

    def create_xml(self, cr, uid, ids, datas, context={}):
        pool=pooler.get_pool(cr.dbname)
        wizard = pool.get(datas['model']).browse(cr, uid, ids[0])

        report = etree.Element("report")

        config = etree.SubElement(report, "config")
        etree.SubElement(config, 'date').text = '09/09/2005'
        etree.SubElement(config, 'PageSize').text = '210.00mm,297.00mm'
        etree.SubElement(config, 'PageWidth').text = '595.27'
        etree.SubElement(config, 'PageHeight').text = '841.88'
        etree.SubElement(config, 'tableSize').text = '60.00mm,60.00mm,60.00mm'
        etree.SubElement(config, 'report-header').text = 'Resum participacio per grup'
        etree.SubElement(config, 'report-footer').text = 'Generated by Open ERP'
        
        etree.SubElement(report, 'header')

        group_ids=[x.id for x in wizard.group_ids]
        datetime_from=wizard.date_from
        datetime_to=wizard.date_to

        content = etree.SubElement(report, "rfpg")
        etree.SubElement(content, 'date_from').text = datetime_from
        etree.SubElement(content, 'date_to').text = datetime_to
        etree.SubElement(content, 'date-create').text = datetime.now().strftime('%Y-%m-%d %H:%M:%S')


        for group in pool.get('groups.group').browse(cr, uid, group_ids, context={'withoutBlank': False,
                                                                                  'datetime_from': datetime_from, 
                                                                                  'datetime_to': datetime_to}):
            group_node = etree.SubElement(content, 'group')
            etree.SubElement(group_node, 'group_name').text = group.name

            parts=set()
            for ga in group.assignation_ids:
                if ga.datetime_from<datetime_to and ga.datetime_to>datetime_from:
                    parts.add(ga.participation_id)
                parts.add(ga.participation_id)

            course_ids=[]
            for part in parts:
                for x in part.course_lines:
                    if x.course_id.id not in course_ids:
                        course_ids.append(x.course_id.id)
                        etree.SubElement(group_node, 'course').text = x.course_id.code

            for part in sorted(parts, key=lambda x:part.participant.name+', '+part.participant.first_name):
                part_node = etree.SubElement(group_node, 'participation')
                etree.SubElement(part_node, 'name_student').text = part.participant.name
                etree.SubElement(part_node, 'first_name_student').text = part.participant.first_name
                course_data_by_course=dict.fromkeys(course_ids, (0,0))
                for course_line in part.course_lines:
                    course_data_by_course[course_line.course_id.id]=(course_line.seance_hours,course_line.absence_hours)
                for course_id in course_ids:
                    course_data_node = etree.SubElement(part_node, 'course_data')
                    etree.SubElement(course_data_node, 'seance_hours').text = str(course_data_by_course[course_id][0])
                    etree.SubElement(course_data_node, 'absence_hours').text = str(course_data_by_course[course_id][1])

        return self.post_process_xml_data(cr, uid, etree.tostring(report, encoding="UTF-8"), context)

report_custom('report.school.resum.participacio.per.grup', 'groups.group', '', 'addons/school_attendance/reports/resum_participacio_per_grup.xsl')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

