# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv, fields
from datetime import datetime, timedelta
import wizard
import pooler

class school_room(osv.osv):
    _name = 'school.room'
    _inherit = 'school.room'
    _columns = {
        'substitutions' : fields.one2many('school.program_substitutions', 'room_id', string='Program substitutions', help='Program substitutions'),
    }
school_room()


class school_program_substitutions(osv.osv):
    _name = 'school.program_substitutions'
    _columns = {
        'classe_id': fields.many2one('school.classe','Class',select=1,required=True,ondelete='cascade',),
        'room_id': fields.many2one('school.room','Room',select=1,required=True,ondelete='cascade',),
        'sequence': fields.integer('Priority',),
    }
    _defaults = {
        'sequence': lambda *a: 1,
    }
school_program_substitutions()

class school_seance(osv.osv):
    _name = 'school.seance'
    _inherit = 'school.seance'

    def _get_substitutes(self, cr, uid, ids, field_name, arg, context=None):
        ret={}
        for id in ids: ret[id]=[]
        query="SELECT s.id,ts.teacher_id FROM school_seance AS s INNER JOIN school_program_substitutions AS ps ON s.room_id=ps.room_id INNER JOIN school_seance AS s1 ON ps.classe_id=s1.classe_id AND s1.date<=s.date AND s1.date_to >= s.date_to INNER JOIN school_teacher_seance AS ts ON ts.seance_id=s1.id WHERE s.id IN %s"
        cr.execute(query, (tuple(ids),))
        for (id,teacher_subst_id) in cr.fetchall(): ret[id].append(teacher_subst_id)
        return ret

    def _search_by_substitute(self, cr, uid, obj, name,args, context=None):
        print "school_seance: _search_by_substitute: cr, uid, obj, name,args, context: ",cr, uid, obj, name,args, context
        return []

    _columns = {
        'substitutes': fields.function(_get_substitutes, type='many2many', obj='school.teacher', method=True, fnct_search=_search_by_substitute, string="Substitutes",),
    }

school_seance()

class open_seances_to_substitute(wizard.interface):
    def _open_seances_to_substitute(self, cr, uid, data, context=None):
        if not context: context={}
        mod_obj = pooler.get_pool(cr.dbname).get('ir.model.data')
        act_obj = pooler.get_pool(cr.dbname).get('ir.actions.act_window')

        result = mod_obj._get_id(cr, uid, 'school_extension', 'school_seance_list_act')
        id = mod_obj.read(cr, uid, [result], ['res_id'])[0]['res_id']
        result = act_obj.read(cr, uid, [id], [])[0]

        query="""
        SELECT s1.id FROM res_partner AS p
            INNER JOIN school_teacher AS t ON p.id=t.partner_id
            INNER JOIN school_teacher_seance AS ts ON p.id=ts.teacher_id
            INNER JOIN school_seance AS s ON ts.seance_id=s.id
            INNER JOIN school_program_substitutions AS ps ON s.classe_id=ps.classe_id
            INNER JOIN school_seance AS s1 ON ps.room_id=s1.room_id AND s.date<=s1.date AND
                s.date_to >= s1.date_to
        WHERE p.user_id=%s
        """
        cr.execute(query, (uid,))
        res_ids=[]
        for (id,) in cr.fetchall(): res_ids.append(id)
        result['name'] = 'Substitution seances'
        result['domain'] = "[('id','in',%s),('date','>','%s'),('date','<','%s'),('state','!=','closed')]" % (res_ids+[0],(datetime.today()-timedelta(days=7)).strftime('%Y-%m-%d %H:%M:%S'),(datetime.today()+timedelta(days=7)).strftime('%Y-%m-%d %H:%M:%S'))
        result.pop('search_view',False)
        return result

    states = {
          'init': {
                'actions': [],
                'result': {'type': 'action',
                           'action': _open_seances_to_substitute,
                           'state':'end'}
          },
    }
open_seances_to_substitute('school_seances_to_substitute_ow')


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
