# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#      Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#    Copyright (C) 2011 Zikzakmedia S.L. (http://www.zikzakmedia.com)
#      Jesús Martín Jiménez <jmargin@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv, fields, orm
from datetime import datetime, date, timedelta
from tools.translate import _
from timed_groups.timed_groups import _MIN_DATETIME,_MAX_DATETIME

class school_teacher(osv.osv):
    _name = "school.teacher"
school_teacher()

class res_partner(osv.osv):
    _name  =  "res.partner"
    _inherit  =  "res.partner"
    
    _columns  =  {
        'doc_type': fields.char('Doc type', size = 1),
        'teacher_id' : fields.many2one('school.teacher','Teacher',ondelete='cascade',readonly=True,),
    }
    
    def write(self, cr, uid, ids, vals, context = None):
        if not context: context = {}
        change = {'name': 'teacher_name', 'ref': 'teacher_code', 'user_id': 'teacher_user_id'}
        sub_vals = dict(filter(None, map(lambda (x,y): x in change and (change[x],y) or False, vals.items())))
        if sub_vals:
            teacher_ids = [x.teacher_id.id for x in self.browse(cr, uid, ids) if x.teacher_id]
            if teacher_ids:
                context2 = dict(context)
                context2['from_partner'] = True
                self.pool.get('school.teacher').write(cr, uid, teacher_ids, sub_vals, context = context2)
        vals.pop('teacher_id',False)
        return super(res_partner, self).write(cr, uid, ids, vals, context = context)

res_partner()

class school_school(osv.osv):
    _name = 'school.school'
school_school()

class res_users(osv.osv):
    _name  =  "res.users"
    _inherit  =  "res.users"
    _columns  =  {
        'school_id': fields.many2one('school.school','School', select = 1, help = 'School that is related to the current user'),
        'teacher_id' : fields.many2one('school.teacher', 'Teacher', ondelete='cascade',readonly=True,),
    }

    def write(self, cr, uid, ids, vals, context = None):
        if not context: context = {}
        if filter(lambda x: x in ['teacher_id',],vals.keys()):
            raise orm.except_orm('Error!','Write not permitted.')
        change = {'name': 'teacher_name', 'login': 'teacher_code'}
        sub_vals = dict(filter(None, map(lambda (x,y): x in change and (change[x],y) or False, vals.items())))
        if sub_vals:
            teacher_ids = [x.teacher_id.id for x in self.browse(cr, uid, ids) if x.teacher_id]
            if teacher_ids:
                context2 = dict(context)
                context2['from_user'] = True
                self.pool.get('school.teacher').write(cr, uid, teacher_ids, sub_vals, context = context2)
        return super(res_users, self).write(cr, uid, ids, vals, context = context)

res_users()

class hr_employee(osv.osv):
    _name = "hr.employee"
    _inherit  =  "hr.employee"

    _columns = {
        'teacher_id' : fields.many2one('school.teacher', 'Teacher', ondelete='cascade',readonly=True,),
    }

    def write(self, cr, uid, ids, vals, context = None):
        if not context: context = {}
        if filter(lambda x: x in ['teacher_id',],vals.keys()):
            raise orm.except_orm('Error!','Write not permitted.')
        change = {'name': 'teacher_name', }
        sub_vals = dict(filter(None, map(lambda (x,y): x in change and (change[x],y) or False, vals.items())))
        if sub_vals:
            teacher_ids = [x.teacher_id.id for x in self.browse(cr, uid, ids) if x.teacher_id]
            if teacher_ids:
                context2 = dict(context)
                context2['from_employee'] = True
                self.pool.get('school.teacher').write(cr, uid, teacher_ids, sub_vals, context = context2)
        return super(hr_employee, self).write(cr, uid, ids, vals, context = context)

hr_employee()

class res_partner_contact(osv.osv):
    _name  =  "res.partner.contact"
    _inherit  =  "res.partner.contact"

    _columns  =  {
        'doc_type': fields.char('Doc type', size = 1, ),
        'vat': fields.char('VAT',size = 32 ),
        'gender': fields.selection([('M','Man'),('W','Woman')],'Gender'),
    }

res_partner_contact()

class school_student(osv.osv):
    _name = "school.student"
    _inherits = {'res.partner.contact' : 'contact_id',}

    _columns = {
        'contact_id' : fields.many2one('res.partner.contact','Contact',),
    }

school_student()

class school_teacher(osv.osv):
    _name = "school.teacher"
    _rec_name = 'teacher_name'

    _columns = {
        'partner_id' : fields.many2one('res.partner','Partner',ondelete='restrict',readonly=True,),
        'employee_id': fields.many2one('hr.employee','Employee',ondelete='restrict',readonly=True,),
        'teacher_user_id' : fields.many2one('res.users','User',ondelete='restrict',readonly=True,),
        'teacher_name' : fields.char('Name',size=200,required=True,),
        'teacher_code': fields.char('Code',size=30,required=True,),
    }

    def create(self, cr, uid, vals, context = None):
        tid = super(school_teacher, self).create(cr, uid, vals, context = context)
        user_id = self.pool.get('res.users').create(cr, uid, {'teacher_id': tid, 'name': vals['teacher_name'], 'login': vals['teacher_code']}, context = context)
        partner_id = self.pool.get('res.partner').create(cr, uid, {'teacher_id': tid, 'name': vals['teacher_name'], 'ref': vals['teacher_code'], 'user_id': user_id,}, context = context)
        employee_id = self.pool.get('hr.employee').create(cr, uid, {'teacher_id': tid, 'name': vals['teacher_name'], }, context = context)
        vals2 = {}
        vals2['teacher_user_id'] = user_id
        vals2['partner_id'] = partner_id
        vals2['employee_id'] = employee_id
        super(school_teacher, self).write(cr, uid, [tid], vals2)
        return tid
        
    def write(self, cr, uid, ids, vals, context = None):
        if not context: context = {}
        if filter(lambda x: x in ['partner_id','employee_id','teacher_user_id'],vals):
            raise orm.except_orm('Error!','Write not permitted.')
        (partner_ids,employee_ids,user_ids) = zip(*[(x.partner_id.id,x.employee_id.id,x.teacher_user_id.id) for x in self.browse(cr, uid, ids)])
        if not context.get('from_partner',False):
            change = {'teacher_name': 'name', 'teacher_code': 'ref', 'teacher_user_id': 'user_id'}
            vals2 = dict(map(lambda (x,y): (change[x],y), filter(lambda (x,y): x in change, vals.items())))
            if vals2:
                super(res_partner, self.pool.get('res.partner')).write(cr, uid, list(partner_ids), vals2, context = context)
        if not context.get('from_user',False):
            change = {'teacher_name': 'name', 'teacher_code': 'login',}
            vals2 = dict(map(lambda (x,y): (change[x],y), filter(lambda (x,y): x in change, vals.items())))
            if vals2:
                super(res_users, self.pool.get('res.users')).write(cr, uid, list(user_ids), vals2, context = context)
        if not context.get('from_employee',False):
            change = {'teacher_name': 'name', }
            vals2 = dict(map(lambda (x,y): (change[x],y), filter(lambda (x,y): x in change, vals.items())))
            if vals2:
                super(hr_employee, self.pool.get('hr.employee')).write(cr, uid, list(employee_ids), vals2, context = context)
        return super(school_teacher, self).write(cr, uid, ids, vals, context = context)
        
school_teacher()

class school_course(osv.osv):
    _name  =  'school.course'
    
    def name_search(self, cr, uid, name='',args=None, operator='ilike',context=None, limit=80):
        return self.name_get(cr, uid, self.search(cr, uid, ['|',('name','ilike',name),('code','ilike',name)], limit = limit, context = context), context = context)

    _columns  =  {
        'name' : fields.char('Name', size = 100, required = True, select = 1, help = "The course name."),
        'code' : fields.char('Code', size = 10, required = True, select = 1, help = "The course code."),
        'special' : fields.boolean('Special', select = 1, help = "If the course is to put eat outher." ),
        'active' : fields.boolean('Active', select = 1),
        'min_hours' : fields.integer('Min Hours'),
    }

    _defaults  =  {
        'active' : lambda *a: 1,
        'min_hours' : lambda *a : 1,
    }

    _order  =  'name'

school_course()

class school_offer(osv.osv):
    _name = 'school.offer'
school_offer()


class school_course_line(osv.osv):
    _name  =  'school.course.line'
    _columns  =  {
        'course_id' : fields.many2one('school.course', 'Course',ondelete = 'restrict', required = True, select = 1, help = "The course."),
        'offer_id': fields.many2one('school.offer', 'Offer', ondelete = 'restrict', required = True, select = 1, help = "The offer."),
        'weekly_hours' : fields.integer('Weekly hours'),
        'needly' : fields.selection(
            [
                ('required', 'Required'),
                ('optional', 'Optional'),
            ],
            'Needly',
            readonly = False,
            help = "The need of the course into the offer.",
            required = True,
        ),
    }

    _order  =  'needly'
    _defaults  =  {
        'needly' : lambda *a: 'required',
    }

    _sql_constraints  =  [('course_unique_by_offer','unique(course_id,offer_id)','Course couldn''t repeat in a offer')]

    def name_get(self, cr, uid, ids):
        ret = {}
        for id in ids: ret[id] = ''
        query = """
SELECT scl.id, sc.name || ' (' || so.name || ')'
FROM school_course_line as scl
 INNER JOIN school_offer AS so ON scl.offer_id = so.id
 INNER JOIN school_course AS sc ON scl.course_id = sc.id
WHERE scl.id IN %s"""
        cr.execute(query,(tuple(ids),))
        for (id,name) in cr.fetchall(): ret[id] = name
        # @type ret dict
        return ret.items()

    def name_search(self, cr, uid, name = '', args = [], operator = 'ilike', context = None, limit = 80):
        res = []
        query = """
SELECT scl.id, sc.name || ' (' || so.name || ')'
FROM school_course_line as scl
 INNER JOIN school_offer AS so ON scl.offer_id = so.id
 INNER JOIN school_course AS sc ON scl.course_id = sc.id
WHERE sc.name || ' (' || so.name || ')' ilike '%%%s%%'"""
        cr.execute(query,(name,))
        data = cr.fetchall()
        ids = zip(data)[0]
        ids2 = self.search(cr, uid, [('id','in',ids)]+args, limit = limit, context = context)
        for tupla in data:
            if tupla[0] in ids2: res.append(tupla)
        return res

school_course_line()


class school_offer(osv.osv):
    _name = 'school.offer'

    _columns = {
        'name' : fields.char('Name', size=32, required=True, select=1, help="The offer name"),
        'courses_lines' : fields.one2many('school.course.line','offer_id','Courses',help='List of the courses in the offer'),
        'active' : fields.boolean('Active', select=2,),
        'session_ids' : fields.one2many('school.session','offer',string='Sessions',),
    }

    _defaults = {
        'active' : lambda *a: 1,
    }

    _order = 'name'

school_offer()


class school_session(osv.osv):
    _name = 'school.session'

    
    _columns = {
        'name' : fields.char('Name', size=32, required=True, select=1, help="The session name"),
        'offer' : fields.many2one('school.offer','Offer',select=1,help="The offer relationed with the item",),
        'date_from' : fields.date('Date from', required=True,),
        'date_to' : fields.date('Date to', required=True,),
        'participants' : fields.one2many('groups.participation','session_id', ),
        'active' : fields.boolean('Active', select=2,),
        'school_id': fields.many2one('school.school', 'School', select = 1, help = 'School that is related to the current session.'),
    }

    _defaults = {
        'active' : lambda *a: 1,
    }

    _sql_constraints = [
        ('dates_ok','CHECK (date_to>=date_from)','Date to minor than date from'),
        ('name_uniq', 'unique (name)', 'The Name of the Session must be unique !')
    ]

    _order = 'name'

    def copy(self, cr, uid, id, default = {}, context = None):
        session_name = self.read(cr, uid, [id], ['name'])[0]['name']
        default.update({
            'name': _('%s (copy)') % session_name,
        })
        return super(school_session, self).copy(cr, uid, id, default, context)

school_session()

class school_holidays_calendar(osv.osv):
    _name = 'school.holidays_calendar'
school_holidays_calendar()

class school_holiday_interval(osv.osv):
    _name = 'school.holiday_interval'
    _columns = {
        'name' : fields.char('Name', size=30,),
        'date_from' : fields.date('Date from', select=1, required=True,),
        'date_to' : fields.date('Date to', select=1, required=True,),
        'calendars' : fields.many2many('school.holidays_calendar','school_holidays_calendar_intervals_rel','days_id','cal_id',string="Calendars"),
    }

    def _check_date_from_to(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        return obj.date_from <= obj.date_to

    _constraints = [
        (_check_date_from_to, "Please, check the start date !", ['date_from', 'date_to']),
    ]

    _sql_constraints = [
        ('uniq_holiday_interval', 'unique(name)', 'The holiday interval of the year must be unique !'),
    ]

school_holiday_interval()

class school_holidays_calendar(osv.osv):
    _name = 'school.holidays_calendar'
    _columns = {
        'name' : fields.char('Name', size=30,),
        'holidays' : fields.many2many('school.holiday_interval','school_holidays_calendar_intervals_rel','cal_id','days_id',string="Holidays"),
        'school_id': fields.many2one('school.school', 'School', select = 1, help = 'School that is related to the current calendar.'),
    }
    
    _sql_constraints = [
        ('uniq_holiday_calendar', 'unique(name)', 'The holiday calendar must be unique !'),
    ]

school_holidays_calendar()

class school_school(osv.osv):
    _name = 'school.school'
    _columns = {
        'name': fields.char('Name', size = 30, required = True, select = 1),
        'partner_id': fields.many2one('res.partner', 'Partner', required = True, select = 1, help = 'Partner that is related to the current school.'),
        'user_ids': fields.one2many('res.users', 'school_id', string = 'Users', help = 'Users of this school.'),
        'calendar_ids': fields.one2many('school.holidays_calendar', 'school_id', string = 'Calendars', help = 'Calendars of this school.'),
        'session_ids': fields.one2many('school.session', 'school_id', string = 'Sessions', help = 'Sessions of this school.'),
    }

school_school()

class school_params(osv.osv):
    _name = 'school.params'
    _description = "To inherit and to add module parameters."

    _columns = {
        'name' : fields.char('',size=6,readonly=True,),
    }

    def default_get(self, cr, uid, fields=None, context=None):
        ids=self.search(cr, uid, [('id','!=',0)])
        if ids:
            return super(school_params, self).read(cr, uid, ids, fields or [], context=context)[0]
        ret={}.fromkeys(fields or [], False)
        ret['name']=''
        return ret

    def create(self, cr, uid, vals, context=None):
        ids=self.search(cr, uid, [('id','!=',0)])
        if ids:
            return super(school_params, self).write(cr, uid, ids, vals, context=context)
        return super(school_params, self).create(cr, uid, vals, context=context)

    def get_value(self, cr, uid, param, context=None):
        ret=False
        for data in self.read(cr, uid, self.search(cr, uid, [('id','!=',0)]), [param]):
            ret=data[param]
        return ret
school_params()



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
