# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#      Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#    Copyright (C) 2011 Zikzakmedia S.L. (http://www.zikzakmedia.com)
#      Jesús Martín Jiménez <jmargin@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from timed_groups.timed_groups import _MIN_DATETIME,_MAX_DATETIME

class groups_group(osv.osv):
    _name = 'groups.group'
    _inherit = 'groups.group'

    def _limits(self, cr, uid, ids, field_name, arg, context = None):
        res = {}
        for g in self.browse(cr, uid, ids):
            id = g.id
            res[id] = {
                'limit_from' : _MAX_DATETIME,
                'limit_to' : _MIN_DATETIME}
            for ga in g.assignation_ids:
                if res[id]['limit_from'] == _MAX_DATETIME or res[id]['limit_from']>ga.limit_from:
                    res[id]['limit_from'] = ga.limit_from
                if res[id]['limit_to'] == _MIN_DATETIME or res[id]['limit_to'] < ga.limit_to:
                    res[id]['limit_to'] = ga.limit_to
        return res

    def _group_ids(self, cr, uid, ids, context = None):
        res = set()
        for item in self.pool.get('groups.group_assignation').read(cr, uid, ids, ['group_id']):
            res.add(type(item['group_id']) == type(tuple()) and item['group_id'][0] or item['group_id'])
        return list(res)

    _columns = {
        'limit_from' : fields.function(_limits, type='datetime', method=True, string='Limit from', multi='limits',store={
            'groups.group_assignation': (_group_ids, ['datetime_from','datetime_to','participation_id'], 10),
            },),
        'limit_to' : fields.function(_limits, type='datetime', method=True, string='Limit to', multi='limits',store={
            'groups.group_assignation': (_group_ids, ['datetime_from','datetime_to','participation_id'], 10),
            },),
    }
        
groups_group()

class groups_group_assignation(osv.osv):
    _name = 'groups.group_assignation'
    _inherit = 'groups.group_assignation'

    def _limits(self, cr, uid, ids, field_name, arg, context=None):
        res={}
        for ga in self.browse(cr, uid, ids,context={'without_blank': False}):
            id=ga.id
            res[id]={}
            part_limit_from=ga.participation_id.session_id.date_from or _MIN_DATETIME[0:10]
            res[id]['limit_from']=part_limit_from+" 00:00:00"
            if ga.datetime_from:
                res[id]['limit_from']=max(ga.datetime_from,res[id]['limit_from'])
            part_limit_to=ga.participation_id.session_id.date_to or _MAX_DATETIME[0:10]
            res[id]['limit_to']=part_limit_to+" 23:59:59"
            if ga.datetime_to:
                if ga.datetime_to<res[id]['limit_to']: res[id]['limit_to']=ga.datetime_to
        return res

    def _group_assignations_from_session(self, cr, uid, ids, context=None):
        return self.pool.get('groups.group_assignation').search(cr, uid, [('participation_id.session_id','in',ids)])
        
    _columns =  {
        'limit_from' : fields.function(_limits, type = 'datetime', method = True, string = 'Limit from', select = True, multi = 'limits',store = {
            'school.session': (_group_assignations_from_session, ['date_from','date_to'], 20),
            'groups.group_assignation': (lambda self, cr, uid, ids, context = None: ids, ['datetime_from','datetime_to','participation_id'], 10),
            },),
        'limit_to' : fields.function(_limits, type = 'datetime', method = True, string = 'Limit to', select = True, multi = 'limits',store = {
            'school.session': (_group_assignations_from_session, ['date_from','date_to'], 20),
            'groups.group_assignation': (lambda self, cr, uid, ids, context = None: ids, ['datetime_from','datetime_to','participation_id'], 10),
            },),
    }

groups_group_assignation()

class groups_participation(osv.osv):
    _name = 'groups.participation'
    _inherit = 'groups.participation'


    def name_get(self, cr, uid, ids, context = None):
        res = []
        for item in self.browse(cr, uid, ids):
            session_name = ''
            if item.session_id:
                session_name = item.session_id.name
            res.append((item['id'],'%s - %s, %s (%s)' % (session_name, item.participant.name, item.participant.first_name, item.name)))
        return res

    def name_search(self, cr, uid, name = '', args = None, operator = 'ilike', context = None, limit = 80):
        contact_ids = self.pool.get('res.partner.contact').search(cr, uid, [('name','ilike',name)], limit = limit, context = context)
        session_ids = self.pool.get('school.session').search(cr, uid, [('name','ilike',name)], limit = limit, context = context)
        ids = self.search(cr, uid, ['|',('participant','in',contact_ids),('session_id','in',session_ids)], limit = limit, context = context)
        return self.name_get(cr, uid, ids, context = context)

    def _data(self, cr, uid, ids, field_name, arg, context = None):
        ret = {}
        for id in ids: ret[id] = {'date_from': False, 'date_to': False, }
        query = """
select
    ga.participation_id,min(ga.datetime_from) as date_from,
    max(ga.datetime_to) as date_to
from groups_group_assignation as ga
where participation_id in %s group by ga.participation_id
"""
        cr.execute(query, (tuple(ids),))
        for (id,date_from,date_to) in cr.fetchall():
            if date_from > _MIN_DATETIME: ret[id]['date_from'] = date_from
            if date_to < _MAX_DATETIME: ret[id]['date_to'] = date_to
        return ret

    def _get_part_name(self, cr, uid, ids, field_name, arg, context = None):
        ret = {}
        for item in self.browse(cr, uid, ids):
            session_name = ''
            if item.session_id:
                session_name = item.session_id.name
            ret[item.id] = '%s - %s, %s (%s)' % (session_name, item.participant.name, item.participant.first_name, item.name)
        return ret

    _columns = {
        'date_from': fields.function(_data, type = 'datetime', multi = 'data', method = True,),
        'date_to' : fields.function(_data, type = 'datetime', multi = 'data', method = True,),
        'session_id' : fields.many2one('school.session', 'Session', ondelete = 'cascade', ),
        'ref' : fields.char('Ref', size = 30,select = 1,),
        'part_name' : fields.function(_get_part_name, type='char', size=150, string = 'Name', method = True, store = {
            'res.partner.contact': (lambda self,cr,uid,ids,context={}: self.pool.get('groups.participation').search(cr, uid, [('participant','in',ids)]), ['name','first_name'], 10),
            'group.participation' : (lambda self,cr,uid,ids,context={}: ids, ['participant'], 20)}),
    }

groups_participation()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
