# -*- encoding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school modules
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import pooler
from datetime import datetime, timedelta
import copy
from lxml import etree
from report.interface import report_rml
from report.interface import toxml
from comu_resum_faltes import get_part_data,_DISCARD_ABSENCES_SAME_TIME_CONTEXT_KEY

#FIXME: we should use toxml

class report_custom(report_rml):

    def create_xml(self, cr, uid, ids, datas, context={}):
        pool=pooler.get_pool(cr.dbname)
        wizard = pool.get(datas['model']).browse(cr, uid, ids[0])
        obj_ga = pool.get('groups.group_assignation')

        report = etree.Element("report")

        config = etree.SubElement(report, "config")
        etree.SubElement(config, 'date').text = '09/09/2005'
        etree.SubElement(config, 'PageSize').text = '210.00mm,297.00mm'
        etree.SubElement(config, 'PageWidth').text = '595.27'
        etree.SubElement(config, 'PageHeight').text = '841.88'
        etree.SubElement(config, 'tableSize').text = '60.00mm,60.00mm,60.00mm'
        etree.SubElement(config, 'report-header').text = 'Faltes per participacio'
        etree.SubElement(config, 'report-footer').text = 'Generated by Open ERP'
        
        etree.SubElement(report, 'header')

        part_ids=[x.id for x in wizard.participation_ids]
        groups_ids=[x.id for x in wizard.group_ids]
        datetime_from=wizard.date_from
        datetime_to=wizard.date_to

        if groups_ids:
            ga_ids=obj_ga.search(cr, uid, [('group_id','in',groups_ids),('datetime_from','<',datetime_to),('datetime_to','>',datetime_from)])
            if ga_ids:
                part_ids+=[x for x in [type(x['participation_id'])==type([]) and x['participation_id'][0] or x['participation_id'] for x in obj_ga.read(cr, uid, ga_ids, ['participation_id'])] if x not in part_ids]

        content = etree.SubElement(report, "rfpg")
        etree.SubElement(content, 'date_from').text = datetime_from
        etree.SubElement(content, 'date_to').text = datetime_to
        etree.SubElement(content, 'date-create').text = datetime.now().strftime('%Y-%m-%d %H:%M:%S')

        def passa_a_hores(td):
            return (td.seconds + td.days * 24 * 3600)/3600

        context2=copy.deepcopy(context)
        context2[_DISCARD_ABSENCES_SAME_TIME_CONTEXT_KEY]=True
        for (part_id,part_data) in get_part_data(cr, uid, part_ids, datetime_from, datetime_to, context=context2).items():
            part_node = etree.SubElement(content, 'participation', id=str(part_id))
            etree.SubElement(part_node,'name_student').text = part_data['name']
            etree.SubElement(part_node,'first_name_student').text = part_data['first_name']
            etree.SubElement(part_node,'tutor_name').text = part_data['tutor_name']
            part_line = etree.SubElement(part_node,'line')
            for (ldate,stat) in part_data['by_date'].items():
                if stat['absent'] or stat['delayed'] or stat['incidence'] or stat['expulsion']:
                        etree.SubElement(part_line,'date').text = ldate
                        etree.SubElement(part_line,'num_seances').text = str(len(stat['id_sessions']))
                        etree.SubElement(part_line,'time_seances').text = str(passa_a_hores(stat['time_seances']))
                        etree.SubElement(part_line,'absences').text = str(stat['absent'])
                        etree.SubElement(part_line,'absences_j').text = str(stat['absent_j'])
                        etree.SubElement(part_line,'absences_time').text = str(passa_a_hores(stat['absent_time']))
                        etree.SubElement(part_line,'absences_time_j').text = str(passa_a_hores(stat['absent_time_j']))
                        etree.SubElement(part_line,'delays').text = str(stat['delayed'])
                        etree.SubElement(part_line,'delays_j').text = str(stat['delayed_j'])
                        etree.SubElement(part_line,'incidences').text = str(stat['incidence'])
                        etree.SubElement(part_line,'expulsions').text = str(stat['expulsion'])
            stat=part_data['total']
            etree.SubElement(part_node,'t_num_seances').text = str(len(stat['id_sessions']))
            etree.SubElement(part_node,'t_time_seances').text = str(passa_a_hores(stat['time_seances']))
            etree.SubElement(part_node,'t_absences').text = str(stat['absent'])
            etree.SubElement(part_node,'t_absences_j').text = str(stat['absent_j'])
            etree.SubElement(part_node,'t_absences_time').text = str(stat['absent_time'])
            etree.SubElement(part_node,'t_absences_time_j').text = str(stat['absent_time_j'])
            etree.SubElement(part_node,'t_delays').text = str(stat['delayed'])
            etree.SubElement(part_node,'t_delays_j').text = str(stat['delayed_j'])
            etree.SubElement(part_node,'t_incidences').text = str(stat['incidence'])
            etree.SubElement(part_node,'t_expulsions').text = str(stat['expulsion'])

        return self.post_process_xml_data(cr, uid, etree.tostring(report, encoding="UTF-8"), context)

report_custom('report.school.resum.faltes.per.participacio', 'groups.participation', '', 'addons/school_extension/reports/resum_faltes_per_participacio.xsl')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

