# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from timed_groups.timed_groups import _MIN_DATETIME,_MAX_DATETIME

class school_participation_course_data(osv.osv):
    _name = 'school.participation_course_data'
    _inherit = 'school.participation_course_data'

    
    def _compute_seance_hours(self, cr, uid, ids, field_name, arg, context=None):
        if not context: context={}
        ret={}
        for pcd_id in ids:
            ret[pcd_id]={'seance_hours': 0,}
        query="""
        SELECT
            pcd.id,
            sum(extract(epoch from least(s.date_to,ga.datetime_to,%(datetime_to)s)-greatest(s.date,ga.datetime_from,%(datetime_from)s))/3600) as seance_hours
        FROM school_participation_course_data AS pcd
        INNER JOIN groups_group_assignation AS ga ON ga.participation_id=pcd.participation_id
        INNER JOIN school_classe AS c ON c.course_id=pcd.course_id
        INNER JOIN school_seance AS s ON s.classe_id=c.id AND s.group_id=ga.group_id AND
            s.state = 'closed' AND
            s.date<ga.datetime_to AND
            s.date_to > ga.datetime_from AND
            s.date<%(datetime_to)s AND s.date_to>%(datetime_from)s
        WHERE pcd.id IN %(pcd_ids)s
        GROUP BY pcd.id
        ORDER BY pcd.id
                """
        cr.execute(query, {'pcd_ids': tuple(ids), 'datetime_from': context.get('datetime_from',_MIN_DATETIME), 'datetime_to': context.get('datetime_to',_MAX_DATETIME)} )
        for (pcd_id,seance_hours) in cr.fetchall():
            ret[pcd_id]['seance_hours']+=seance_hours
        return ret

    _columns = {
        'seance_hours' : fields.function(_compute_seance_hours, method=True, string='Seance hours', type='float', multi='compute',),
    }
        
    
school_participation_course_data()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
