# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#    Copyright (C) 2011 Zikzakmedia S.L. (http://www.zikzakmedia.com)
#       Jesús Martín Jiménez <jmargin@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

_TABLA_USERS_GROUPS = 'res_groups_users_rel'
_GROUP_TUTOR_FUNCTION_CODE = 'GT'
_TUTOR_FUNCTION_CODES = ['GT']
_TUTORS_GROUP_NAME = 'School Tutor group'
USER_ADMIN = 1

class groups_group(osv.osv):
    _name = 'groups.group'
    _inherit = 'groups.group'

    _columns = {
        'tutors' : fields.many2many('res.partner','school_group_tutors_rel','group_id','tutor_id',string="Tutors",),
    }

    def _actualitza_tutors_de_grup(self, cr, uid, tutors_before, tutors_after):
        function_id = self.pool.get('contact.annotation.function').search(cr, uid, [('code','=',_GROUP_TUTOR_FUNCTION_CODE)])[0]
        contact_per_group = {}
        for (group_id,tutor_id) in tutors_after ^ tutors_before: # symmetric difference
            contact_per_group[group_id] = []
        if contact_per_group:
            query = "SELECT DISTINCT ga.group_id,p.participant FROM groups_group_assignation AS ga INNER JOIN groups_participation AS p ON ga.participation_id=p.id WHERE ga.group_id IN %s"
            cr.execute(query, (tuple(contact_per_group.keys()),))
            for (group_id,contact_id) in cr.fetchall(): contact_per_group[group_id].append(contact_id)
        for (group_id,tutor_id) in tutors_after - tutors_before:
            cr.execute("SELECT contact_id FROM contact_partner_function AS cpf WHERE cpf.function_id=%s AND partner_id=%s AND contact_id IN %s", (function_id,tutor_id,tuple(contact_per_group[group_id])))
            tutor_existent = []
            for (tutor,) in cr.fetchall(): tutor_existent.append(tutor)
            for contact_id in contact_per_group[group_id]:
                if contact_id not in tutor_existent:
                    self.pool.get('contact.partner.function').create(cr, uid, {'contact_id': contact_id, 'partner_id': tutor_id, 'function_id': function_id})
        for (group_id,tutor_id) in tutors_before - tutors_after:
            cr.execute("SELECT p.participant FROM groups_group_assignation AS ga INNER JOIN groups_participation AS p ON ga.participation_id=p.id INNER JOIN school_group_tutors_rel AS gt ON gt.tutor_id=%s WHERE gt.group_id<>%s AND participant IN %s" % (tutor_id,group_id,tuple(contact_per_group[group_id])))
            contacts_to_remains = []
            for (contact_id,) in cr.fetchall(): contacts_to_remains.append(contact_id)
            cpf_to_unlink = self.pool.get('contact.partner.function').search(cr, uid, [('partner_id','=',tutor_id),('function_id','=',function_id),('contact_id','in',[x for x in contact_per_group[group_id] if x not in contacts_to_remains]),])
            self.pool.get('contact.partner.function').unlink(cr, uid, cpf_to_unlink)

    def write(self, cr, uid, ids, vals, context=None):
        ids2 = type(ids) in (int,long) and [ids] or ids
        tutors_before=set()
        tutors_after=set()
        if 'tutors' in vals:
            query = "SELECT group_id,tutor_id FROM school_group_tutors_rel WHERE group_id IN %s"
            cr.execute(query, (tuple(ids2),) )
            for (group_id,tutor_id) in cr.fetchall(): tutors_before.add( (group_id,tutor_id) )
        ret=super(groups_group, self).write(cr, uid, ids, vals, context=context)
        if 'tutors' in vals:
            query = "SELECT group_id,tutor_id FROM school_group_tutors_rel WHERE group_id IN %s"
            cr.execute(query, (tuple(ids2),) )
            for (group_id,tutor_id) in cr.fetchall(): tutors_after.add( (group_id,tutor_id) )
            self._actualitza_tutors_de_grup(cr, uid, tutors_before, tutors_after)
        return ret
        
    def create(self, cr, uid, vals, context=None):
        ret = super(groups_group, self).create(cr, uid, vals, context=context)
        if 'tutors' in vals and type(vals['tutors'])==type([]):
            tutors_after=set()
            query = "SELECT group_id,tutor_id FROM school_group_tutors_rel WHERE group_id=%s"
            cr.execute(query, (ret,) )
            for (group_id,tutor_id) in cr.fetchall(): tutors_after.add( (group_id,tutor_id) )
            self._actualitza_tutors_de_grup(cr, uid, set(), tutors_after)
        return ret
        
    def unlink(self, cr, uid, ids, context=None):
        ids2 = type(ids) in (int,long) and [ids] or ids
        tutors_before=set()
        query = "SELECT group_id,tutor_id FROM school_group_tutors_rel WHERE group_id IN %s"
        cr.execute(query, (tuple(ids2),) )
        for (group_id,tutor_id) in cr.fetchall(): tutors_before.add( (group_id,tutor_id) )
        self._actualitza_tutors_de_grup(cr, uid, tutors_before, set())
        return super(groups_group, self).unlink(cr, uid, ids, context=context)
        
groups_group()

class contact_partner_function(osv.osv):
    _name = 'contact.partner.function'
    _inherit = 'contact.partner.function'
        
    def write(self, cr, uid, ids, vals, context = None):
        ids2 = type(ids) in (int,long) and [ids] or ids
        tutor_ids = set([x.partner_id.id for x in self.browse(cr, uid, ids2) if x.partner_id and x.function_id.code in _TUTOR_FUNCTION_CODES])
        ret=super(contact_partner_function, self).write(cr, uid, ids, vals, context=None)
        tutor_ids.update([x.partner_id.id for x in self.browse(cr, uid, ids2) if x.partner_id and x.function_id.code in _TUTOR_FUNCTION_CODES])
        if tutor_ids:
            self.deixa_ok_funcio_tutor(cr, USER_ADMIN, tutor_ids, context = context)
        return ret
        
    def create(self, cr, uid, vals, context = None):
        ret = super(contact_partner_function, self).create(cr, uid, vals, context = None)
        if 'partner_id' in vals:
            self.deixa_ok_funcio_tutor(cr, USER_ADMIN, [vals['partner_id']], context = context)
        return ret
        
    def unlink(self, cr, uid, ids, context = None):
        ids2 = type(ids) in (int,long) and [ids] or ids
        tutor_ids = set([x.partner_id.id for x in self.browse(cr, uid, ids2) if x.partner_id and x.function_id.code in _TUTOR_FUNCTION_CODES])
        ret = super(contact_partner_function, self).unlink(cr, uid, ids, context = None)
        if tutor_ids:
            self.deixa_ok_funcio_tutor(cr, USER_ADMIN, tutor_ids, context = context)
        return ret
        
    def deixa_ok_funcio_tutor(self, cr, uid, tutors_tocats = None, context = None):
        res_group_ids = self.pool.get('res.groups').search(cr, uid, [('name','=',_TUTORS_GROUP_NAME)])
        if not res_group_ids: return

        usuaris_per_tutor = {}
        tots_els_usuaris = set()
        if tutors_tocats is None:
            cr.execute("SELECT id,user_id FROM res_partner WHERE user_id IS NOT NULL")
        else:
            cr.execute("SELECT id,user_id FROM res_partner WHERE id IN %s AND user_id IS NOT NULL", (tuple(tutors_tocats),))            
        for (tutor_id,user_id) in cr.fetchall():
            usuaris_per_tutor[tutor_id] = user_id
            tots_els_usuaris.add(user_id)
        
        usuaris_en_grup_tutor_abans = set()
        if tots_els_usuaris:
            cr.execute("SELECT uid FROM "+_TABLA_USERS_GROUPS+" WHERE gid=%s AND uid IN %s", (res_group_ids[0], tuple(tots_els_usuaris)))
            for (user_id,) in cr.fetchall(): usuaris_en_grup_tutor_abans.add(user_id)

        tutors_tocats_que_ara_tenen_funcio = set()
        function_ids = self.pool.get('contact.annotation.function').search(cr, uid, [('code','in',_TUTOR_FUNCTION_CODES)])
        if function_ids:
            cr.execute("SELECT partner_id FROM contact_partner_function AS cpf WHERE cpf.function_id IN %s AND partner_id IN %s", (tuple(function_ids),tuple(tutors_tocats)))
            for (tutor_id,) in cr.fetchall(): tutors_tocats_que_ara_tenen_funcio.add(tutor_id)
        
        usuaris_en_grup_tutor_ara = set([usuaris_per_tutor[x] for x in tutors_tocats_que_ara_tenen_funcio if x in usuaris_per_tutor])
        
        usuaris_a_treure_de_grup_tutors = usuaris_en_grup_tutor_abans-usuaris_en_grup_tutor_ara
        if usuaris_a_treure_de_grup_tutors:
            cr.execute("DELETE FROM "+_TABLA_USERS_GROUPS+" WHERE gid=%s AND uid IN %s", (res_group_ids[0], tuple(usuaris_a_treure_de_grup_tutors)))
        
        usuaris_a_afegir_al_grup_tutors = usuaris_en_grup_tutor_ara-usuaris_en_grup_tutor_abans
        for user_id in usuaris_a_afegir_al_grup_tutors:
            cr.execute("INSERT INTO "+_TABLA_USERS_GROUPS+" (gid,uid) VALUES (%s,%s)", (res_group_ids[0],user_id))

    def comprova_tots(self, cr, uid, context = None):
        self.deixa_ok_funcio_tutor(cr, uid, None, context = context)
        
contact_partner_function()

class contact_annotation(osv.osv):
    _name = 'contact.annotation'
    _inherit = 'contact.annotation'
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context = {}
        if context.get('only_tutorizeds', False):
            function_id = self.pool.get('contact.annotation.function').search(cr, uid, [('code','=',_GROUP_TUTOR_FUNCTION_CODE)])[0]
            contact_args , rest_args = [], []
            # contact args by_pass
            for (field,op,value) in args:
                (field.split('.',1)[0] == 'contact_id' and contact_args or rest_args).append( (field,op,value) )
            cpf_ids = self.pool.get('contact.partner.function').search(cr, uid, contact_args + [('function_id','=',function_id),('partner_id.user_id','=',uid)])
            args = rest_args + [ ('contact_id','in',[cpf_item.contact_id.id for cpf_item in self.pool.get('contact.partner.function').browse(cr, uid, cpf_ids)] ) ]
        return super(contact_annotation, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=False)
                
contact_annotation()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
