import bisect
from datetime import datetime

from osv import fields
from osv import orm
from osv import osv
from tools.translate import _

class school_impartition_week_line(osv.osv):
    _name = 'school.impartition_week_line'
    _inherit = 'school.impartition_week_line'

    def _compute_blocked(self, cr, uid, ids, field_name, arg, context=None):
        ret = {}
        for iwl in self.browse(cr, uid, ids):
            blocked = True
            ret[iwl.id] = {'needed_complete': (iwl.teachers_needed <= len(iwl.teachers)), }
            for td in iwl.teachers:
                blocked &= td.blocked
                if not blocked: break
            ret[iwl.id]['teachers_blocked'] = blocked
        return ret

    def _get_iwl_ids_from_td_ids(self, cr, uid, ids, context=None):
        ret = set()
        for td in self.browse(cr, uid, ids):
            ret.add(td.iwl_id.id)
        return list(ret)


    _columns = {
        'teachers_blocked': fields.function(_compute_blocked, type='boolean', method=True, string='Blocked', multi='teachers', help='No teachers changes allowed',
                                            store={'school.teacher_data': (_get_iwl_ids_from_td_ids, ['blocked'], 10)},
                                            ),
        'iwl_group': fields.char('IWL group', size=30, help="Group name for IWL's with same teacher", ),
        'teachers_needed': fields.integer('Teachers needed', ),
        'teachers_recommended': fields.integer('Teachers recommended', ),
        'needed_completed': fields.function(_compute_blocked, type='boolean', method=True, string='Teachers completed', multi='teachers',
                                            store={'school.teacher_data': (_get_iwl_ids_from_td_ids, ['iwl_id', ], 10)},
                                            ),
    }

    _defaults = {
        'teachers_needed': lambda * a: 1,
        'teachers_recommended': lambda * a: 1,
    }

    _sql_constraints = [
        ('needed_and_recommended_check', 'CHECK (teachers_needed <= teachers_recommended)', "The number required should be smaller than the recommended"),
        ('needed_not_negative', 'CHECK (teachers_needed >= 0)', "The number required should be not negative"),
        ]

school_impartition_week_line()

class school_teacher_data(osv.osv):
    _name = 'school.teacher_data'
    _inherit = 'school.teacher_data'

    _columns = {
        'blocked': fields.boolean('Blocked', help='No teachers changes allowed',),
    }

school_teacher_data()

class school_block_iwl_wizard(osv.osv_memory):
    _name = 'school.block_iwl_wizard'

    _columns = {
        'iwl_ids': fields.many2many('school.impartition_week_line', 'iwl_block_wizard_rel', 'wizard_id', 'iwl_id', string='IWLs', ),
        'block': fields.boolean('Block', help='Block, yes or no',),

    }

    _defaults = {
        'iwl_ids': lambda self, cr, uid, context = {}: [(6,0,context.get('active_ids', []))],
    }

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_block(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids):
            iwl_ids = [x.id for x in item.iwl_ids]
            td_ids = self.pool.get('school.teacher_data').search(cr, uid, [('iwl_id', 'in', iwl_ids)])
            self.pool.get('school.teacher_data').write(cr, uid, td_ids, {'blocked': item.block, })

school_block_iwl_wizard()

class school_block_td_wizard(osv.osv_memory):
    _name = 'school.block_td_wizard'

    _columns = {
        'td_ids': fields.many2many('school.impartition_week_line', 'iwl_block_wizard_rel', 'wizard_id', 'iwl_id', string='IWLs', ),
        'block': fields.boolean('Block', help='Block, yes or no',),

    }

    _defaults = {
        'td_ids': lambda self, cr, uid, context = {}: [(6,0,context.get('active_ids', []))],
    }

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_block(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids):
            td_ids = [x.id for x in item.td_ids]
            self.pool.get('school.teacher_data').write(cr, uid, td_ids, {'blocked': item.block, })

school_block_td_wizard()

class school_teachers_solution(osv.osv):
    _name = 'school.teachers_solution'

school_teachers_solution()

class school_teacher_iwl_solution(osv.osv):
    _name = 'school.teacher_iwl_solution'
    
    _columns = {
        'solution_id': fields.many2one('school.teachers_solution', 'Solution', ondelete='cascade', required=True, ),
        'iwl_id': fields.many2one('school.impartition_week_line', 'IWL', ondelete='cascade', required=True, ),
        'teacher_id': fields.many2one('school.teacher', 'Teacher', ondelete='cascade', required=True, ),
        'value': fields.float('Value', ),
    }
    
school_teacher_iwl_solution()

class school_teachers_solution(osv.osv):
    _name = 'school.teachers_solution'
    
    def _compute(self, cr, uid, ids, field_name, arg, context=None):
        return sum([x.percentage for x in self.browse(cr, uid, ids, context=context)]) / len(ids)
        
    _columns = {
        'name': fields.char('Name', size=100, ),
        'created': fields.datetime('Created', ),
        'total_hours': fields.float('Total hours', ),
        'total_assigned': fields.float('Total assigned', ),
        'teacher_iwl_ids': fields.one2many('school.teacher_iwl_solution', 'solution_id', 'Solution', ),
        'value': fields.float('Value', ),
    }
    
    _defaults = {
        'created': lambda * a: datetime.today().strftime('%Y-%m-%d %H:%M:%S'),
    }
school_teachers_solution()

def agrupa_cursos_i_professors(teacher_data_by_course):
    cursos = {}
    professors = {}
    ret = []

    for (curs, _dicci) in teacher_data_by_course.items():
        for _dicci2 in _dicci['list']:
            professor = _dicci2['id']
            if curs not in cursos:
                if professor not in professors:
                    dicci = {'courses': set([curs]), 'professors': set([professor]), }
                    cursos[curs] = len(ret)
                    professors[professor] = len(ret)
                    ret.append(dicci)
                else:
                    dicci = ret[professors[professor]]
                    dicci['courses'].add(curs)
                    cursos[curs] = professors[professor]
            else:
                if professor not in professors:
                    dicci = ret[cursos[curs]]
                    dicci['professors'].add(professor)
                    professors[professor] = cursos[curs]
                else:
                    if cursos[curs] == professors[professor]:
                        continue
                    dicci = ret[cursos[curs]]
                    dicci2 = ret[professors[professor]]
                    dicci['courses'].update(dicci2['courses'])
                    dicci['professors'].update(dicci2['professors'])
                    for curs2 in dicci2['courses']:
                        cursos[curs2] = cursos[curs]
                    for professor2 in dicci2['professors']:
                        professors[professor2] = cursos[curs]
                    dicci2['courses'] = False
                    dicci2['professors'] = False
    return [x for x in ret if x['courses']]

class school_solution_data_by_course(osv.osv_memory):
    _name = 'school.solution.data_by_course'
    
    _columns = {
        'solution_id' : fields.many2one('school.create_solutions','Solution'),
        'course_id' : fields.many2one('school.course','Course',),
        'classes_hours' : fields.float('Classes Hours'),
        'max_hours' : fields.float('Max hours'),
        'min_hours' : fields.float('Min hours'),
    }
    
school_solution_data_by_course()

class school_create_solutions_wizard(osv.osv_memory):
    _name = 'school.create_solutions'
    
    _columns = {
        'number_max_solutions': fields.integer('Max. solutions'),
        'total_classes_hours': fields.float('Total classes hours', readonly=True, ),
        'total_max_hours': fields.float('Max teachers hours', readonly=True, ),
        'total_min_hours': fields.float('Min teachers hours', readonly=True, ),
        'data_by_course' : fields.one2many('school.solution.data_by_course', 'solution_id', readonly=True, )
    }

    def _default_total_classes_hours(self, cr, uid, context=None):
        (total_classes_hours, classes_data_by_course) = self.get_classes_data_by_course(cr, uid)
        return total_classes_hours

    def _default_total_max_hours(self, cr, uid, context=None):
        (total_classes_hours, classes_data_by_course) = self.get_classes_data_by_course(cr, uid)
        (total_teachers_hours, data_by_teacher, teachers_data_by_course) = self.get_teachers_data_by_course(cr, uid, classes_data_by_course.keys())
        return total_teachers_hours['pool_max']

    def _default_total_min_hours(self, cr, uid, context=None):
        (total_classes_hours, classes_data_by_course) = self.get_classes_data_by_course(cr, uid)
        (total_teachers_hours, data_by_teacher, teachers_data_by_course) = self.get_teachers_data_by_course(cr, uid, classes_data_by_course.keys())
        return total_teachers_hours['pool_min']

    def _default_data_by_course(self, cr, uid, context=None):
        (total_classes_hours, classes_data_by_course) = self.get_classes_data_by_course(cr, uid)
        (total_teachers_hours, data_by_teacher, teachers_data_by_course) = self.get_teachers_data_by_course(cr, uid, classes_data_by_course.keys())
        ret = []
        for (course_id, classes_data) in classes_data_by_course.items():
            ret.append({0: -1,'course_id': course_id,
                        'classes_hours': classes_data['duration'],
                        'max_hours': teachers_data_by_course[course_id]['pool_max'],
                        'min_hours': teachers_data_by_course[course_id]['pool_min'],})

        return ret
            
    _defaults = {
        'total_classes_hours': _default_total_classes_hours,
        'total_max_hours': _default_total_max_hours,
        'total_min_hours': _default_total_min_hours,
        'data_by_course' : _default_data_by_course,
    }

    def get_teachers_hours_blocked(self, cr, uid):
        obj_td = self.pool.get('school.teacher_data')
        total_hours = 0
        hours_by_teacher = {}
        hours_by_course_and_teacher = {}
        
        td_ids = obj_td.search(cr, uid, [('blocked', '=', True)])
        for td in obj_td.browse(cr, uid, td_ids):
            total_hours += td.iwl_id.duration
            if td.teacher_id.id not in hours_by_teacher:
                hours_by_teacher[td.teacher_id.id] = 0
            hours_by_teacher[td.teacher_id.id] += td.iwl_id.duration
            course_id = td.iwl_id.classe_id.course_id.id
            key = (course_id, td.teacher_id.id)
            if key not in hours_by_course_and_teacher:
                hours_by_course_and_teacher[key] = 0
            hours_by_course_and_teacher[key] += td.iwl_id.duration
        return (total_hours, hours_by_teacher, hours_by_course_and_teacher)
        
    
    
    def get_classes_data_by_course(self, cr, uid):
        obj_iwl = self.pool.get('school.impartition_week_line')
        iwl_ids = obj_iwl.search(cr, uid, [('teachers_blocked', '=', False), ])
        total_hours = 0
        ret = {}
        classes = {}
        for iwl in obj_iwl.browse(cr, uid, iwl_ids):
            classe_id = iwl.classe_id.id
            course_id = iwl.classe_id.course_id.id
            if not course_id in ret:
                ret[course_id] = {'duration': 0, 'classes': []}
                classes[course_id] = {}
            if (classe_id, iwl.iwl_group) not in classes[course_id]:
                classes[course_id][(classe_id, iwl.iwl_group)] = {'duration': 0, 'iwls': []}
            classes[course_id][(classe_id, iwl.iwl_group)]['duration'] += iwl.duration
            classes[course_id][(classe_id, iwl.iwl_group)]['iwls'].append(iwl.id)
            ret[course_id]['duration'] += iwl.duration
            total_hours += iwl.duration
        for course_id in ret.keys():
            ret[course_id]['classes'] = list(classes[course_id].values())
        return (total_hours, ret)

    

    def get_teachers_data_by_course(self, cr, uid, courses):
        total_hours = {'pool_max': 0, 'pool_min': 0}
        (total_blocked_hours, blocked_hours_by_teacher, blocked_hours_by_course_and_teacher) = self.get_teachers_hours_blocked(cr, uid)
        
        obj_tcs = self.pool.get('school.teacher_course_suitability')
        teachers_suitability_ids = obj_tcs.search(cr, uid, [('course_id', 'in', list(courses))])

        teacher_ids = set()
        teachers_data_by_course = {}
        for item in obj_tcs.browse(cr, uid, teachers_suitability_ids):
            teacher_ids.add(item.teacher_id.id)
            key = (item.course_id.id, item.teacher_id.id)
            if item.course_id.id not in teachers_data_by_course:
                teachers_data_by_course[item.course_id.id] = {'pool_max': 0, 'pool_min': 0, 'list': [], }
            max_hours = max(0, item.max_week_hours - blocked_hours_by_course_and_teacher.get(key, 0))
            min_hours = max(0, item.min_week_hours - blocked_hours_by_course_and_teacher.get(key, 0))
            teachers_data_by_course[item.course_id.id]['list'].append ({
                                                                       'percentage': item.percentage, 
                                                                       'id': item.teacher_id.id,
                                                                       'pool_max': max_hours,
                                                                       'pool_min': min_hours, })
            teachers_data_by_course[item.course_id.id]['pool_max'] += max_hours
            teachers_data_by_course[item.course_id.id]['pool_min'] += min_hours

        obj_t = self.pool.get('school.teacher')
        data_by_teacher = {}
        for x in obj_t.browse(cr, uid, obj_t.search(cr, uid, [('id','in',list(teacher_ids))])):
            max_hours = max(0, x.max_week_hours - blocked_hours_by_teacher.get(x.id, 0))
            min_hours = max(0, x.min_week_hours - blocked_hours_by_teacher.get(x.id, 0))
            if max_hours > 0:
                data_by_teacher[x.id] = {
                    'pool_max': max_hours,
                    'pool_min': min_hours,
                }
                total_hours['pool_max'] += max_hours
                total_hours['pool_min'] += min_hours
        
        return (total_hours, data_by_teacher, teachers_data_by_course)

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def create_solutions(self, cr, uid, ids, context=None):
        
        sols_from_wizard = []
        for create_solutions_wizard in self.browse(cr, uid, ids):
            number_max_solutions = create_solutions_wizard.number_max_solutions
            (total_classes_hours, classes_data_by_course) = self.get_classes_data_by_course(cr, uid)
            (total_teachers_hours, data_by_teacher, teachers_data_by_course) = self.get_teachers_data_by_course(cr, uid, classes_data_by_course.keys())
            
            errors = []
            if total_classes_hours > total_teachers_hours['pool_max']: errors.append( _('Total hours to assign exceeds teachers disponibility.') )
            if total_classes_hours < total_teachers_hours['pool_min']: errors.append( _('No classe hours for minimum requires') )
            for (course_id, classe_data) in classes_data_by_course.items():
                if classe_data['duration'] > teachers_data_by_course[course_id]['pool_max']:
                    errors.append( _('Total hours to assign exceeds teachers disponibility by course with id %s.') % (course_id,) )
                if classe_data['duration'] < teachers_data_by_course[course_id]['pool_min']:
                    errors.append( _('No classe hours for minimum requires by course with id %s.') % (course_id,) )
            if errors:
                raise orm.except_orm('Error!', '\n'.join(errors))
                                
            for agrupacio in agrupa_cursos_i_professors(teachers_data_by_course):
                def assigna_classes(classes, data_by_teacher, teacher_course_data_list, total_hours, total_hours2, total_min_hours, total_min_hours2, max_solutions, classe_cursor=0, afinament = 2):
                    llista_de_solucions_a_retornar = []
                    classe_duration = classes[classe_cursor]['duration']                    

                    tcd_list_ordered = sorted(teacher_course_data_list, key=lambda x: (x['pool_min'],data_by_teacher[x['id']]['pool_min'],x['percentage'],x['pool_max']), reverse = True)
                    new_list = []
                    max_hours = 0
                    for tcd in tcd_list_ordered:
                        # Reduim la llista de professors
                        # Deixem els que tenen un minim d'hores per aquest curs
                        # i els que necessiten aquest curs per cumplir el minim d'hores generals
                        # completem amb els restants fins a tenir un nombre d'hores mes que suficient
                        if tcd['pool_min'] > 0 or max_hours < total_hours * afinament:
                            new_list.append(tcd)
                            max_hours += tcd['pool_max']
                        else:
                            break
                    for punter_teacher in range(len(new_list)):
                        teacher = new_list[punter_teacher]
                        if teacher['id'] not in data_by_teacher: continue
                        check_max_hours = teacher['pool_max'] > classe_duration
                        check_max_hours2 = data_by_teacher[teacher['id']]['pool_max'] > classe_duration
                        check_min_hours = (total_min_hours - teacher['pool_min'] <= total_hours - classe_duration)
                        check_min_hours2 = (total_min_hours2 - data_by_teacher[teacher['id']]['pool_min'] <= total_hours2 - classe_duration)
                        if check_max_hours and check_min_hours and check_max_hours2 and check_min_hours2:
                            # TODO : Mirar en FET
                            # classes[classe_cursor]['teacher'] = teacher['id']
                            # mira en FET
                            
                            # Apuntem la solucio actual
                            if len(llista_de_solucions_a_retornar) < max_solutions or llista_de_solucions_a_retornar[-1] < teacher['percentage']:
                                new_max = (teacher['percentage'], [teacher['id']])
                                bisect.insort(llista_de_solucions_a_retornar, new_max)
                                if len(llista_de_solucions_a_retornar) > max_solutions:
                                    llista_de_solucions_a_retornar = llista_de_solucions_a_retornar[0:max_solutions]

                            # Si es la darrera classe no cal fer mes recursivitat
                            if classe_cursor + 1 >= len(classes):
                                continue

                            # Actualitzem les variables de les estructures de dades
                            teacher['pool_max'] -= classe_duration
                            data_by_teacher[teacher['id']]['pool_max'] -= classe_duration
                            min_to_reduce = min(classe_duration, teacher['pool_min'])
                            teacher['pool_min'] -= min_to_reduce
                            min_to_reduce2 = min(classe_duration, data_by_teacher[teacher['id']]['pool_min'])
                            data_by_teacher[teacher['id']]['pool_min'] -= min_to_reduce2

                            # Executem la recursio
                            value_assignacio_list = assigna_classes(classes, data_by_teacher, teacher_course_data_list, total_hours - classe_duration, total_hours2 - classe_duration, total_min_hours - min_to_reduce, total_min_hours2 - min_to_reduce2, max_solutions, classe_cursor = classe_cursor + 1)

                            # Deixem les variables de les estructures de dades com estaven
                            teacher['pool_max'] += classe_duration
                            teacher['pool_min'] += min_to_reduce
                            data_by_teacher[teacher['id']]['pool_max'] += classe_duration
                            data_by_teacher[teacher['id']]['pool_min'] += min_to_reduce2

                            # Apuntem els resultats de la recursio
                            for (value, assignacio) in value_assignacio_list:
                                if len(llista_de_solucions_a_retornar) < max_solutions or llista_de_solucions_a_retornar[max_solutions-1][0] < value + teacher['percentage']:
                                    new_max = (value + teacher['percentage'], [teacher['id']] + assignacio)
                                    llista_de_solucions_a_retornar.reverse()
                                    bisect.insort(llista_de_solucions_a_retornar, new_max)
                                    llista_de_solucions_a_retornar.reverse()
                            if len(llista_de_solucions_a_retornar) > max_solutions:
                                llista_de_solucions_a_retornar = llista_de_solucions_a_retornar[0:max_solutions]
                                        
                    return llista_de_solucions_a_retornar

                def assigna_cursos(cursos_list_orig, total_classes_hours, classes_data_by_course, total_teachers_hours, data_by_teacher, teachers_data_by_course, max_solutions):
                    llista_de_solucions_a_retornar = []
                    for courses_punter in range(len(cursos_list_orig)):
                        cursos_list = cursos_list_orig[:courses_punter] + cursos_list_orig[courses_punter + 1:]
                        course_id = cursos_list_orig[courses_punter]

                        classes = classes_data_by_course[course_id]['classes']
                        total_course_duration = classes_data_by_course[course_id]['duration']
                        teacher_course_data = teachers_data_by_course[course_id]
                        teacher_course_data_list = teacher_course_data['list']

                        sols = assigna_classes(classes, data_by_teacher, teacher_course_data_list, total_course_duration, total_classes_hours, teacher_course_data['pool_min'], total_teachers_hours['pool_min'], max_solutions)

                        if not sols: continue

                        if not cursos_list:
                            for (value, assignacio) in sols:
                                llista_de_solucions_a_retornar.append((value, {course_id: assignacio, }))

                            continue

                        for (value, assignacio) in sols:
                            # Contem les hores per reduir el necessari en l'estructura
                            # de dades d'hores maximes que passarem per assignar els seguents cursos
                            hours_by_teacher = {}
                            total_hours = 0
                            for i in range(len(assignacio)):
                                teacher_id = assignacio[i]
                                classe = classes[i]
                                if teacher_id not in hours_by_teacher:
                                    hours_by_teacher[teacher_id] = 0
                                hours_by_teacher[teacher_id] += classe['duration']
                                total_hours += classe['duration']

                            # Apliquem la reduccio i contabilitzem les hores minimes
                            total_teachers_hours['pool_max'] -= total_hours
                            min_to_reduce = 0
                            min_to_reduce_by_teacher = {}
                            for (teacher_id, hours) in hours_by_teacher.items():
                                data_by_teacher[teacher_id]['pool_max'] -= hours
                                min_to_reduce_by_teacher['teacher_id'] = min(hours, data_by_teacher[teacher_id]['pool_min'])
                                data_by_teacher[teacher_id]['pool_min'] -= min_to_reduce_by_teacher['teacher_id']
                                min_to_reduce += min_to_reduce_by_teacher['teacher_id']
                            total_teachers_hours['pool_min'] -= min_to_reduce

                            # Fem recursivitat per
                            value_assignacio_list = assigna_cursos(cursos_list, total_classes_hours - total_hours, classes_data_by_course, total_teachers_hours, data_by_teacher, teachers_data_by_course, max_solutions)
                            total_teachers_hours['pool_max'] += total_hours
                            total_teachers_hours['pool_min'] += min_to_reduce
                            for (teacher_id, hours) in hours_by_teacher.items():
                                data_by_teacher[teacher_id]['pool_max'] += hours
                                data_by_teacher[teacher_id]['pool_min'] += min_to_reduce_by_teacher['teacher_id']

                            for (value2, assignacio_by_course) in value_assignacio_list:
                                if len(llista_de_solucions_a_retornar) < max_solutions or llista_de_solucions_a_retornar[max_solutions-1][0] < value2 + value:
                                    assignacio_by_course[course_id] = assignacio
                                    new_max = (value2 + value, assignacio_by_course)
                                    llista_de_solucions_a_retornar.reverse()
                                    bisect.insort(llista_de_solucions_a_retornar, new_max)
                                    llista_de_solucions_a_retornar.reverse()
                            if len(llista_de_solucions_a_retornar) > max_solutions:
                                llista_de_solucions_a_retornar = llista_de_solucions_a_retornar[0:max_solutions]
                    return llista_de_solucions_a_retornar
                                    
                # TODO: fer una llista de cursos ordenada comencant per el mes dificil d'assignar
                #       professors. Es provaran totes pero s'ha de mirar de comencar a
                #       permutar per on hi ha mes probabilitat de solucio
                sols = assigna_cursos(list(agrupacio['courses']), total_classes_hours, classes_data_by_course, total_teachers_hours, data_by_teacher, teachers_data_by_course, number_max_solutions)

                # Passar solucions a base de dades
                for (value, assignacio_by_course) in sols:
                    sol_id = self.pool.get('school.teachers_solution').create(cr, uid, {'value': value, 'total_hours': total_classes_hours, })
                    sols_from_wizard.append(sol_id)
                    total_assigned = 0
                    for (course_id, assignacio) in assignacio_by_course.items():
                        for i in range(len(assignacio)):
                            classe = classes_data_by_course[course_id]['classes'][i]
                            total_assigned += classe['duration']
                            for iwl_id in classe['iwls']:
                                value = [x['percentage'] for x in teachers_data_by_course[course_id]['list'] if x['id'] == assignacio[i]][0]
                                self.pool.get('school.teacher_iwl_solution').create(cr, uid, {'solution_id': sol_id, 'iwl_id': iwl_id, 'teacher_id': assignacio[i], 'value': value})
                    self.pool.get('school.teachers_solution').write(cr, uid, [sol_id], {'total_assigned': total_assigned, })

        cr.execute('select id,name from ir_ui_view where model=%s and type=%s', ('school.teachers_solution', 'tree'))
        view_res = cr.fetchone()

        return {
            'domain': "[('id','in',%s),]" % (tuple(sols_from_wizard), ),
            'name': _("Solutions from wizard"),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'school.teachers_solution',
            'view_id': view_res,
            'context': context,
            'type': 'ir.actions.act_window',
        }

                    
school_create_solutions_wizard()

class school_apply_teacher_solution(osv.osv_memory):
    _name = 'school.apply_teacher_solution'

    _columns = {
        'solution_ids': fields.many2many('school.teachers_solution', 'teacher_solution_apply_rel', 'wizard_id', 'solution_id', string='Solutions', ),
    }

    _defaults = {
        'solution_ids': lambda self, cr, uid, context = {}: context.get('active_ids', []),
    }

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_change(self, cr, uid, ids, context=None):
        obj_td = self.pool.get('school.teacher_data')
        ids_td_created = []
        ids_to_unlink = []
        for item in self.browse(cr, uid, ids):
            iwls_cleaned = set()
            for solution in item.solution_ids:
                for teacher_iwl in solution.teacher_iwl_ids:
                    iwl_id = teacher_iwl.iwl_id.id
                    if iwl_id not in iwls_cleaned:
                        iwls_cleaned.add(iwl_id)
                        ids_to_unlink += obj_td.search(cr, uid, [('iwl_id','=',iwl_id),('blocked','=',False)])
                    ids_td_created.append( obj_td.create(cr, uid, {'iwl_id': iwl_id, 'teacher_id': teacher_iwl.teacher_id.id, 'title': 'needed'}) )
        obj_td.unlink(cr, uid, ids_to_unlink)

        cr.execute('select id,name from ir_ui_view where model=%s and type=%s', ('school.teacher_data', 'tree'))
        view_res = cr.fetchone()

        return  {
            'domain': "[('id','in',%s),]" % (tuple(ids_td_created), ),
            'name': _("Assignations created by the apply solutions wizard"),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'school.teacher_data',
            'view_id': view_res,
            'context': context,
            'type': 'ir.actions.act_window',
        }
school_apply_teacher_solution()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
