# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2011 Tecnoba S.L. (http://www.tecnoba.com)
#      Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from tools.translate import _


class create_session(osv.osv_memory):
    _name = 'school.create_session'

    _columns = {
        'name' : fields.char('Session name', size = 200,),
        'offer_id' : fields.many2one('school.offer', string='Offer',required=True,help="Offer for the session"),
        'date_from' : fields.date('Date from', required=True,),
        'date_to' : fields.date('Date to', required=True,),
        'num_groups' : fields.integer('Number of groups', required = True,)
    }

    _defaults = {
        'offer_id' : lambda self, cr, uid, context={} : 'active_ids' in context and context['active_ids'] and type(context['active_ids'])==list and context['active_ids'][0] or False
    }

    def _check_dates(self, cr, uid, ids, context=None):
        return [ x for x in self.browse(cr, uid, ids, context=context) if x.date_to > x.date_from ]

    def action_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_apply(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids, context=context):
            self.pool.get('school.session').create(cr, uid, {'name': item.name, 'offer': item.offer_id.id, 'date_from': item.date_from, 'date_to': item.date_to})
            for n_group in range(item.num_groups):
                group_name = "%s%s" % (item.name,chr(65+n_group))
                group_id = self.pool.get('groups.group').create(cr, uid, {'name': group_name})
                for course_line in item.offer_id.courses_lines:
                    course = course_line.course_id
                    classe_name = "%s-%s" % (course.name, group_name)
                    classe_id = self.pool.get('school.classe').create(cr, uid, {'name': classe_name, 'course_id': course.id, 'date_from': item.date_from, 'date_to': item.date_to, 'group_id': group_id})
                    for n_week_hour in range(course_line.weekly_hours):
                        self.pool.get('school.impartition_week_line').create(cr, uid, {'classe_id': classe_id,})
        return {'type': 'ir.actions.act_window_close'}

    _constraints = [(_check_dates, _("To date must be greater than From date"), ['date_from','date_to']),]

create_session()
