# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields, orm
from timed_groups.timed_groups_extension import selection_methods

class school_teacher_course(osv.osv_memory):
    _name = 'school.iwl.handling.teacher_course'

    _columns = {
        'teacher_id' : fields.many2one('school.teacher','Teacher',required=True,),
        'course_id' : fields.many2one('school.course','Course',required=True,),
        'wizard_id' : fields.many2one('school.iwl.handling','Wizard',),
        'course_domain' : fields.char('Course domain',size=200,),
    }
school_teacher_course()

class school_classe(osv.osv):
    _name = 'school.classe'
    _inherit = 'school.classe'

    def reduce_group_level(self, cr, uid, ids, context=None):
        iwl_obj=self.pool.get('school.impartition_week_line')
        cl_obj=self.pool.get('school.classe')
        # Reduim l'ambit de la classe si es possible desdoblant les classes en els subgrups
        for classe in cl_obj.browse(cr, uid, ids):
            data_groups={}
            for iwl in classe.impartition_week_line_ids:
                group_id=iwl.group_id.id
                if group_id not in data_groups: data_groups[group_id]={'duration': 0, 'iwl_ids': [],}
                data_groups[group_id]['duration']+=iwl.duration
                data_groups[group_id]['iwl_ids'].append(iwl.id)
            equal_duration_for_every_group=reduce(lambda x,y: x==y,map(lambda x: x['duration'],data_groups.values()))
            disjoint_iwl_groups=disjoint_groups_check(cr, data_groups.keys())
            if equal_duration_for_every_group and disjoint_iwl_groups:
                for group_id,dicci in data_groups.items():
                    new_classe_id=cl_obj.create(cr, uid, {'group_id': group_id, 'course_id': classe.course_id.id, 'date_from': classe.date_from, 'date_to': classe.date_to, })
                    iwl_obj.write(cr, uid, dicci['iwl_ids'], {'classe_id': new_classe_id, 'subgroup': False,} )
                cl_obj.unlink(cr, uid, [classe.id])

    # Elevem el grup de la classe al grup pare
    def change_group(self, cr, uid, ids, classe_ids, the_group_id, context=None):
        if not context: context={}
        iwl_obj=self.pool.get('school.impartition_week_line')
        cl_obj=self.pool.get('school.classe')
        the_group_name=self.pool.get('groups.group').browse(cr, uid, the_group_id).name

        all_children_ids=set([the_group_id])
        condition=True
        query="SELECT child_id FROM groups_groups_rel WHERE parent_id IN %s"
        while condition:
            cr.execute(query,(tuple(all_children_ids),))
            new_set=set([x for (x,) in cr.fetchall()])
            new_set.update(all_children_ids)
            condition=len(new_set)!=len(all_children_ids)
            all_children_ids=new_set
        # Anem a tractar les classes
        # Remarquem el grup classe com a grup de les línies horaries que no tenen el grup classe per defecte
        for classe in cl_obj.browse(cr, uid, classe_ids):
            if not set([iwl.group_id.id for iwl in classe.impartition_week_line_ids]).issubset(all_children_ids):
                raise orm.except_orm('I not could continue!','A IWL group isn''t subgroup of parent group.')
            iwl_obj.write(cr, uid, [iwl.id for iwl in classe.impartition_week_line_ids if not iwl.subgroup], {'subgroup': classe.group_id.id})
            # Elevem els grups de les classe al grup pare per totes les classes. 
            # Totes les línies horaries de les classes tenen que tenir un grup que sigui subgrup d'aquest grup pare.
            cl_obj.write(cr, uid, classe_ids, {'name': '%s - %s' % (classe.course_id.name, the_group_name),'group_id': the_group_id})
    
school_classe()

def disjoint_groups_check(cr, group_ids):
    if len(group_ids)==1: return True
    for group_id in group_ids:
        temp_ids=list(group_ids)
        temp_ids.remove(group_id)
        while temp_ids and group_id not in temp_ids:
            query="SELECT parent_id FROM groups_groups_rel WHERE child_id IN %s"
            cr.execute(query,(tuple(temp_ids),))
            temp_ids=[x for (x,) in cr.fetchall() if x]
        if temp_ids:
            return False
    return True

class school_iwl_handling(osv.osv_memory):
    _name = 'school.iwl.handling'

    def _get_default_parent_group_domain(self, cr, uid, context=None):
        if not context: context={}
        iwl_ids=context.get('active_ids',[])
        ret=self.on_change_iwl_ids(cr, uid, None, [(6,0,iwl_ids)], context=None)
        return ret['value']['parent_group_domain']

    _columns = {
        'iwl_ids' : fields.many2many('school.impartition_week_line','school_iwl_handling_rel','wizard_id','iwl_id',string='IWLs',),
        'teacher_course_ids' : fields.one2many('school.iwl.handling.teacher_course','wizard_id',string='Teacher course assignation',),
        'state' : fields.selection([('wrong','Wrong'),('ready','Ready'),('done','Done')],string='State',),
        'comment' : fields.char('Comment',size=255,),
        'n_activities' : fields.integer('Num activities',required=True,),
        'classification_name' : fields.char('Classification name',size=200,required=True,),
        'parent_group' : fields.many2one('groups.group','Parent group',),
        'classification_id' : fields.many2one('groups.classification','Classification',),
        'parent_group_domain' : fields.char('Parent Group Domain',size=255,),
        'classification_id_domain' : fields.char('Classification Domain',size=255,),
        'classification_method' : fields.selection(selection_methods,string='Classification method'),
    }

    _defaults = {
        'iwl_ids' : lambda self,cr,uid,context={} : context.get('active_ids',[]),
        'parent_group_domain' : _get_default_parent_group_domain,
        'classification_id_domain' : lambda *a: '0',
        'classification_method' : lambda *a: 'manual',
    }

    def _get_classification_list(self, cr, n_activities, parent_id):
        if not parent_id: return []
        query="SELECT classification FROM groups_groups_rel WHERE classification IS NOT NULL AND parent_id=%(parent_id)s GROUP BY classification HAVING count(*)=%(n_activities)s"
        cr.execute(query, {'parent_id': parent_id, 'n_activities': n_activities,})
        return [x for (x,) in cr.fetchall()]

    def _get_parent_group_list(self, cr, group_ids):
        if not group_ids: return []
        parent_ids=[]
        if len(group_ids)==1:
            parent_ids=list(group_ids)
        else:
            # Si és classificació i només son els grups groups llavors també posa el grup pare com a possible pare dels grups
            # TODO: 
            # Hi ha més d'un grup i per tant haurem de trobar-ne un d'existent amb el número de classificacions que ens demanen o bé fer un grup unió i una nova classificació
            # Busquem si hi ha un fill comú amb cap més pare
            query="""
            SELECT parent_id
            FROM groups_groups_rel
            WHERE classification IN (
                    SELECT classification
                    FROM groups_groups_rel
                    WHERE child_id IN %(group_ids)s )
            GROUP BY parent_id
            HAVING COUNT(*)=%(len_group_ids)s
            """
            cr.execute(query,{'group_ids': tuple(group_ids),'len_group_ids': len(group_ids),})
            parent_ids=[x for (x,) in cr.fetchall() if x]
        return parent_ids


    def on_change_compute_classification(self, cr, uid, ids, n_activities, parent_id, context=None):
        if not context: context={}
        classification_list=self._get_classification_list(cr, n_activities, parent_id)
        return {'value': {'classification_name': False,'classification_method': 'manual','classification_id': False, 'classification_id_domain': ','.join(map(str,classification_list)+['0'])},}

    def on_change_classification(self, cr, uid, ids, classification, context=None):
        if not context: context={}
        item=self.pool.get('groups.classification').browse(cr, uid, classification)
        return {'value': {'classification_name': item.name,'classification_method': item.method,},}


    def on_change_iwl_ids(self, cr, uid, ids, iwl_ids, context=None):
        iwls=self.pool.get('school.impartition_week_line').browse(cr, uid, iwl_ids[0][2])
        iwl_data=self._get_iwl_data(cr, uid, iwls)
        if iwl_data['error']:
            ret_state='wrong'
            comment=iwl_data['comment']
            parent_ids=[]
        else:
            ret_state='ready'
            comment=''
            groups=iwl_data['groups']
            parent_ids=self._get_parent_group_list(cr, groups.keys())
        return {'value': {'state': ret_state, 'comment': comment, 'parent_group': False, 'classification_name': False, 'classification_id': False, 'classification_id_domain': False,'parent_group_domain': ','.join(map(str,parent_ids)+['0'])},}

    def act_cancel(self, cr, uid, ids, context=None):
        pass

    def _get_iwl_data(self, cr, uid, iwl_items):
        duration=-1
        time_courses={}
        groups={}
        classe_ids=[]
        classe_course={}
        error=False
        comment=''
        for item in iwl_items:
            course_id=item.classe_id.course_id.id
            if item.classe_id.id not in classe_ids: classe_ids.append(item.classe_id.id)
            if not course_id in classe_course:
                classe_course[course_id]=item.classe_id.id
            if duration==-1:
                duration=item.duration
            else:
                if duration!=item.duration:
                    error=True
                    comment='The duration for IWL must be equals.'
            group_id=item.group_id.id
            groups[group_id]=item.group_id.name

            if course_id not in time_courses: time_courses[course_id]={}
            if group_id not in time_courses[course_id]: time_courses[course_id][group_id]=0
            time_courses[course_id][group_id]+=1

        # The time courses for each group must be equals
        for (course_id,groups_dict) in time_courses.items():
            # @type groups_dict dict
            for group_id in groups.keys(): groups_dict.setdefault(group_id, 0)
            time_course_group=-1
            for group_id in groups.keys():
                if time_course_group==-1:
                    time_course_group=groups_dict.get(group_id,0)
                else:
                    if time_course_group!=groups_dict.get(group_id,0):
                        error=True
                        comment='The time courses for each group must be equals.'

        # Grups disjunts : cap arriba a ser fill de cap d'ells
        if not disjoint_groups_check(cr, groups.keys()):
            error=True
            comment='A IWL group is subgroup than other.'

        return {
            'groups' : groups,
            'duration' : duration,
            'classe_ids' : classe_ids,
            'classe_course' : classe_course,
            'time_courses' : time_courses,
            'error' : error,
            'comment' : comment,
        }

    def handling(self, cr, uid, ids, context=None):
        iwl_obj=self.pool.get('school.impartition_week_line')
        cl_obj=self.pool.get('school.classe')
        for wizard in self.browse(cr, uid, ids):
            # Get IWL data
            iwl_data=self._get_iwl_data(cr, uid, wizard.iwl_ids)
            if iwl_data['error']:
                raise orm.except_orm('I not could continue!',iwl_data['comment'])
            groups=iwl_data['groups']
            duration=iwl_data['duration']
            classe_ids=iwl_data['classe_ids']
            classe_course=iwl_data['classe_course']
            time_courses=iwl_data['time_courses']

            # Ampliem els grups classe que calguin
            the_group_id=None
            if len(groups)==1:
                # Només hi ha un grup i no cal fer modificacions
                (the_group_id,the_group_name)=groups.items()[0]
            else:
                if wizard.parent_group:
                    (the_group_id,the_group_name)=(wizard.parent_group.id,wizard.parent_group.name)
                else:
                    # No n'hi ha... doncs el creem
                    the_group_name=','.join(groups.values())
                    list_groups_rel=[]
                    new_classification_id=self.pool.get('groups.classification').create(cr, uid, {'name': the_group_name,})
                    for group_id in groups.keys():
                        list_groups_rel.append((0,0,{'classification': new_classification_id, 'child_id': group_id, 'direction':'up',}))
                    the_group_id=self.pool.get('groups.group').create(cr, uid, {'name': wizard.classification_name, 'children_ids': list_groups_rel, })

                # Elevem el grup de les classes al grup pare
                cl_obj.change_group(cr, uid, ids, classe_ids, the_group_id, context=context)
                
                # Canviem totes les línies horaries a una sola classe per cada matèria
                for classe in cl_obj.browse(cr, uid, classe_ids):
                    iwl_obj.write(cr, uid, [iwl.id for iwl in classe.impartition_week_line_ids], {'classe_id': classe_course[classe.course_id.id]})
                cl_obj.unlink(cr, uid, [x for x in classe_ids if x not in classe_course.values()])

#            import pdb;pdb.set_trace()

            # Crea els subgrups
            subgrup_ids=[]
            if wizard.n_activities==1:
                subgrup_ids=list(groups)
            else:
                classification_id=wizard.classification_id.id
                if not classification_id:
                    new_classification_id=self.pool.get('groups.classification').create(cr, uid, {'name': wizard.classification_name,'method': wizard.classification_method,})
                    for c in range(wizard.n_activities):
                        list_groups_rel=[(0,0,{'parent_id': the_group_id, 'classification': new_classification_id, 'direction': 'down',})]
                        subgrup_id=self.pool.get('groups.group').create(cr, uid, {'name': "%s-%s" % (wizard.classification_name,c),'parent_ids': list_groups_rel,'classification': classification_id, })
                        subgrup_ids.append(subgrup_id)
                else:
                    groups_rel=self.pool.get('groups.groups_rel').search(cr, uid, [('classification','=',classification_id),('parent_id','=',the_group_id),])
                    subgrup_ids=[x.child_id.id for x in self.pool.get('groups.groups_rel').browse(cr, uid, groups_rel)]
                    if len(subgrup_ids)!=wizard.n_activities:
                        raise orm.except_orm('I not could continue!','The classification have got %s, not %s groups marked.' % (len(subgrup_ids),wizard.n_activities))


            # Esborra les línies d'impartició
            iwl_obj.unlink(cr, uid, [x.id for x in wizard.iwl_ids])

            # Crea noves línies d'impartició
            for (course_id,classe_id) in classe_course.items():
                for subgrup_id in subgrup_ids:
                    for c in range(time_courses[course_id].values()[0]):
                        iwl_obj.create(cr, uid, {'classe_id': classe_id, 'subgroup': subgrup_id, 'duration': duration,})

            # Reduim els grups de les classes
            cl_obj.reduce_group_level(cr, uid, classe_course.values(), context=context)
        self.write(cr, uid, ids, {'state': 'done'})

school_iwl_handling()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
