# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from datetime import datetime, timedelta
from tools.translate import _

class school_teacher_changes(osv.osv):
    _name = 'school.teacher_changes'
    
    _columns = {
        'teacher_id_out' : fields.many2one('school.teacher','Teacher out',require=True,readonly=True,),
        'teacher_id_in' : fields.many2one('school.teacher','Teacher in',require=True,readonly=True,),
        'seance_id_from' : fields.many2one('school.seance','Seance from',require=True,readonly=True,),
        'seance_id_to' : fields.many2one('school.seance','Seance to',require=True,readonly=True,),
        'self_changed' : fields.boolean('Self changed',),
    }
    
    _defaults = {
        'self_changed' : False,
    }
school_teacher_changes()

class school_course(osv.osv):
    _name = 'school.course'
    _inherit = 'school.course'

    _columns = {
        'order_for_change_teacher' : fields.integer('Order FST',help='Order for teacher changes',),
    }
school_course()

class school_teacher_seance(osv.osv):
    _name = 'school.teacher_seance'
    _inherit = 'school.teacher_seance'
    
    def _teacher_present(self, cr, uid, ids, field_name, args, context = None):
        result = {}
        for lid in ids:
            result[lid] = False
        cr.execute("""
            SELECT td.id, a.action, a.name FROM school_teacher_seance AS td
                LEFT JOIN school_seance AS s ON td.seance_id = s.id
                LEFT JOIN school_teacher AS t ON td.teacher_id=t.id
                LEFT JOIN hr_attendance AS a ON a.employee_id = t.employee_id AND name = (SELECT max(name) FROM hr_attendance WHERE employee_id = a.employee_id AND name < s.date + 5 * INTERVAL '1 minute')
            WHERE td.id IN %s
            ORDER BY a.name
        """, (tuple(ids),))
        for (td_id,action,moment) in cr.fetchall():
            result[td_id] = action == 'sign_in'
        return result
            
    _columns = {
        'no_problem' : fields.boolean('No problem',),
        'is_present' : fields.function(_teacher_present, type = 'boolean', method = True, string = 'Teacher present', )
    }
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context2=context and dict(context) or {}
        in_context_to_process=context2.pop('search_order_for_change_teacher',False)
        in_context_to_process2=context2.pop('search_seances_for_change_teacher',False)
        in_context_to_process3=context2.pop('search_seances_for_self_assignation',False)
        if not count and in_context_to_process:
            # No filtrem, nomes volem que estigui en un ordre determinat
            ret=super(school_teacher_seance, self).search(cr, uid, args, offset=0, limit=None, order=None, context=context2, count=False)
            if ret:
                query="SELECT ts.id FROM school_teacher_seance AS ts INNER JOIN school_seance AS s ON ts.seance_id=s.id INNER JOIN school_classe AS ca ON s.classe_id=ca.id INNER JOIN school_course AS co ON ca.course_id=co.id WHERE ts.id IN %s ORDER BY co.order_for_change_teacher OFFSET %s"
                params=[tuple(ret),offset]
                if limit:
                    query+=" LIMIT %s"
                    params+=[limit]
                cr.execute(query,params)
                ret=[x for (x,) in cr.fetchall()]
        else:
            if in_context_to_process2 or in_context_to_process3:
                def into_free_hours(free_hours,datetime_from,datetime_to):
                    # retornem cert si l'interval de temps es troba dins d'algun
                    # dins la llista passada com a parametre
                    ret = False
                    for free_hour in free_hours:
                        ret |= datetime_from>=free_hour['date'] and datetime_to<=free_hour['date_to']
                    return ret
                if in_context_to_process3:
                    # Calculem les hores determinades com a disponibles per substituir
                    seance_ids = self.pool.get('school.seance').search(cr, uid, [
                            ('teachers.teacher_id.teacher_user_id','=',uid),
                            ('date_to','>',datetime.today().strftime('%Y-%m-%d %H:%M:%S')),
                            '|',('group_id','=',False),('state','=','suspended')  ])
                    free_hours = self.pool.get('school.seance').read(cr, uid, seance_ids, ['date','date_to'])
                # Filtrem perque nomes surtin les assignacions de professor a
                # sessio en les que el professor no estigui al centre.
                # Tampoc presentem les quines haguem marcat com a 'no problem'
                today_seance_ids = self.pool.get('school.seance').search(cr, uid, [
                    ('state','not in',['suspended']),
                    ('group_id','!=',False),
                    ('date_to','>',datetime.today().strftime('%Y-%m-%d %H:%M:%S')),
                    ('date','<',datetime.today().strftime('%Y-%m-%d'))])
                today_teacher_seance_ids = self.search(cr, uid, [('seance_id','in',today_seance_ids),('no_problem','=',False)])
                td_with_teacher_problem = []
                for td in self.browse(cr, uid, today_teacher_seance_ids):
                    if not td.is_present and \
                            (not in_context_to_process3 or into_free_hours(free_hours,td.seance_id.date,td.seance_id.date_to)):
                        td_with_teacher_problem.append(td.id)
                args+=[('id','in',td_with_teacher_problem)]
            ret=super(school_teacher_seance, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context2, count=count)
            if ret and not count:
                cr.execute("""
                    SELECT ts.id
                    FROM school_teacher_seance AS ts
                        INNER JOIN school_seance AS s ON ts.seance_id=s.id
                    WHERE ts.id IN %s
                    ORDER BY s.date 
                """, (tuple(ret),))
                ret = [ts_id for (ts_id,) in cr.fetchall()]
        return ret

    def read(self, cr, uid, ids, fields, context=None, load='_classic_write'):
        context2=context and dict(context) or {}
        in_context_to_process=context2.pop('search_order_for_change_teacher',False)
        ret=super(school_teacher_seance, self).read(cr, uid, ids, fields, context=context2, load=load)
        if in_context_to_process:
            query="SELECT ts.id,0-COALESCE(co.order_for_change_teacher,0) FROM school_teacher_seance AS ts INNER JOIN school_seance AS s ON ts.seance_id=s.id INNER JOIN school_classe AS ca ON s.classe_id=ca.id INNER JOIN school_course AS co ON ca.course_id=co.id WHERE ts.id IN %s"
            params=[tuple([x['id'] for x in ret]),]
            cr.execute(query,params)
            key_order=dict(cr.fetchall())
            ret=sorted(ret, key= lambda x: key_order[x['id']] )
        return ret


school_teacher_seance()
        
class school_change_teachers_wizard(osv.osv_memory):
    _name = 'school.change_teachers_wizard'

    _columns = {
        'teacher_seance_ids' : fields.many2many('school.teacher_seance','dummy_rel','id1','id2',string='Seances',readonly=True,),
        'teacher_to_change_id' : fields.many2one('school.teacher_seance',string='Teacher to change',context={'search_order_for_change_teacher': True,},),
        'teacher_seances_to_change_ids' : fields.text('domain'),
    }

    def default_get(self, cr, uid, fields, context = None):
        if not context:
            context = {}
        ts_ids = context.get('active_ids',False)
        import pdb;pdb.set_trace()
        ts_result = []
        if ts_ids:
            ts_to_change = self.pool.get('school.teacher_seance').browse(cr, uid, ts_ids[0])
            # sessions a la mateixa hora
            ts_candidat_ids = self.pool.get('school.teacher_seance').search(cr, uid, [
                            ('seance_id.date','<',ts_to_change.seance_id.date_to),
                            ('seance_id.date_to','>',ts_to_change.seance_id.date)
                        ])
            ts_a_presentar = []
            teacher_ids_ocupats = set()
            # pot ser que el professor estigui ocupat en una altra linia
            #     horaria de una altra sessio a la mateixa hora
            for ts_candidat in self.pool.get('school.teacher_seance').browse(cr, uid, ts_candidat_ids):
                if ts_candidat.seance_id.group_id and ts_candidat.seance_id.state != 'suspend':
                    teacher_ids_ocupats.add(ts_candidat.teacher_id.id)
                elif ts_candidat.is_present: # sessio sense grup (possible profe sustitut)
                    ts_a_presentar.append((ts_candidat.id, ts_candidat.teacher_id.id))
            for (ts_id,teacher_id) in ts_a_presentar:
                if teacher_id not in teacher_ids_ocupats:
                    ts_result.append(ts_id)
        return dict((key,value) for (key,value) in {
                    'teacher_seances_to_change_ids': ','.join(map(str,[0]+ts_result)),
                    'teacher_seance_ids': ts_ids,
                    'teacher_to_change_id': len(ts_result)==1 and ts_result[0] or False,
                }.items() if not fields or key in fields)
        
    def change_teacher(self, cr, uid, ids, context=None):
        request_ids_to_send = []
        for item in self.browse(cr, uid, ids):
            if item.teacher_to_change_id:
                self.pool.get('school.teacher_seance').write(cr, uid, [item.teacher_seance_ids[0].id], {'teacher_id': item.teacher_to_change_id.teacher_id.id,})
                seance = item.teacher_seance_ids[0].seance_id
                seance_info = _("Date: %s\nRoom: %s\nGroup: %s\nCourse: %s") % (seance.date,seance.room_id and seance.room_id.name or '-',seance.group_id and seance.group_id.name or '',seance.classe_id.course_id.name)
                # Resquest info to new teacher
                user_obj = item.teacher_to_change_id.teacher_id.teacher_user_id
                body = _('A new seance has been assigned to you:\n') + seance_info
                if user_obj:
                    new_request_id = self.pool.get('res.request').create(cr, uid, {'act_from': uid, 'act_to': user_obj.id, 'name': _('You have a new seance!'), 'body': body,})
                    if new_request_id: request_ids_to_send.append(new_request_id)
                # Request info to teacher changed
                user_obj = item.teacher_seance_ids[0].teacher_id.teacher_user_id
                body = _('Your seance has been changed to other teacher:\n') + seance_info
                if user_obj:
                    new_request_id=self.pool.get('res.request').create(cr, uid, {'act_from': uid, 'act_to': user_obj.id, 'name': _('Your seance has been changed to other teacher!')})
                    if new_request_id: request_ids_to_send.append(new_request_id)
                self.pool.get('school.teacher_changes').create(cr, uid, {'teacher_id_out': item.teacher_seance_ids[0].teacher_id.id, 'teacher_id_in': item.teacher_to_change_id.teacher_id.id, 'seance_id_from': item.teacher_to_change_id.seance_id.id, 'seance_id_to': item.teacher_seance_ids[0].seance_id.id})
        self.pool.get('res.request').request_send(cr, uid, request_ids_to_send)
        return {'type': 'ir.actions.act_window_close'}

    def no_problem(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids):
            self.pool.get('school.teacher_seance').write(cr, uid, [x.id for x in item.teacher_seance_ids], {'no_problem': True,})
        return {'type': 'ir.actions.act_window_close'}

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}


school_change_teachers_wizard()

class school_selfchange_teachers_wizard(osv.osv_memory):
    _name = 'school.selfchange_teachers_wizard'

    _columns = {
        'teacher_seance_ids' : fields.many2many('school.teacher_seance','dummy_rel','id1','id2',string='Seances',readonly=True,),
    }

    def default_get(self, cr, uid, fields, context = None):
        if not context:
            context = {}
        ts_ids = context.get('active_ids',False)
        ts_result = []
        if ts_ids:
            ts_candidat_ids = {}
            for ts_to_change in self.pool.get('school.teacher_seance').browse(cr, uid, ts_ids):
                # sessions a la mateixa hora del professor de l'usuari
                ts_candidat_ids.update(dict.fromkeys( self.pool.get('school.teacher_seance').search(cr, uid, [
                                ('seance_id.date','<=',ts_to_change.seance_id.date),
                                ('seance_id.date_to','>=',ts_to_change.seance_id.date_to),
                                ('teacher_id.teacher_user_id','=',uid),
                        ]) , ts_to_change.id))
            ts_a_presentar = []
            teacher_ids_ocupats = set()
            # pot ser que el professor estigui ocupat en una altra linia
            #     horaria de una altra sessio a la mateixa hora
            for ts_candidat in self.pool.get('school.teacher_seance').browse(cr, uid, ts_candidat_ids.keys()):
                if ts_candidat.seance_id.group_id and ts_candidat.seance_id.state != 'suspend':
                    teacher_ids_ocupats.add(ts_candidat.teacher_id.id)
                elif ts_candidat.is_present: # sessio sense grup (possible profe sustitut)
                    ts_a_presentar.append((ts_candidat.id, ts_candidat.teacher_id.id))
            for (ts_id,teacher_id) in ts_a_presentar:
                if teacher_id not in teacher_ids_ocupats:
                    ts_result.append( ts_candidat_ids[ts_id] )
        return dict((key,value) for (key,value) in {
                    'teacher_seance_ids': ts_result,
                }.items() if not fields or key in fields)
        
    def change_teacher(self, cr, uid, ids, context=None):
        USER_ADMIN = 1
        request_ids_to_send = []
        for item in self.browse(cr, uid, ids):
            for ts_to_change in item.teacher_seance_ids:
                ts_ids = self.pool.get('school.teacher_seance').search(cr, uid, [
                                ('seance_id.date','<=',ts_to_change.seance_id.date),
                                ('seance_id.date_to','>=',ts_to_change.seance_id.date_to),
                                ('seance_id.state','=','suspended'),
                                ('teacher_id.teacher_user_id','=',uid),
                        ])
                ts_ids += self.pool.get('school.teacher_seance').search(cr, uid, [
                                ('seance_id.date','<=',ts_to_change.seance_id.date),
                                ('seance_id.date_to','>=',ts_to_change.seance_id.date_to),
                                ('seance_id.group_id','=',False),
                                ('teacher_id.teacher_user_id','=',uid),
                            ])
                if ts_ids:
                    item2 = self.pool.get('school.teacher_seance').browse(cr, uid, ts_ids[0])
                    user_changed_obj = ts_to_change.teacher_id.teacher_user_id
                    if user_changed_obj:
                        seance = item2.seance_id
                        # Request info to teacher changed
                        body = _('Your seance has been changed to other teacher:\n') +\
                                     _("Date: %s\nRoom: %s\nGroup: %s\nCourse: %s") % (
                                                seance.date,seance.room_id and seance.room_id.name or '-',
                                                seance.group_id and seance.group_id.name or '',
                                                seance.classe_id.course_id.name)
                        new_request_id=self.pool.get('res.request').create(cr, USER_ADMIN, {
                                    'act_from': uid, 
                                    'act_to': user_changed_obj.id, 
                                    'name': _('Your seance has been changed to other teacher!'),
                                    'body': body,   })
                        if new_request_id:
                            request_ids_to_send.append(new_request_id)
                    self.pool.get('school.teacher_seance').write(cr, USER_ADMIN, [ts_to_change.id], {'teacher_id': item2.teacher_id.id,})
                    self.pool.get('school.teacher_changes').create(cr, USER_ADMIN, {
                            'teacher_id_out': ts_to_change.teacher_id.id,
                            'teacher_id_in': item2.teacher_id.id,
                            'seance_id_from': item2.seance_id.id,
                            'seance_id_to': ts_to_change.seance_id.id,
                            'self_changed': True,   })
        self.pool.get('res.request').request_send(cr, USER_ADMIN, request_ids_to_send)
        return {'type': 'ir.actions.act_window_close'}

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}


school_selfchange_teachers_wizard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
