import collections
import bisect
from osv import fields
from osv import osv
from timed_groups.timed_groups import _MAX_DATETIME, _MIN_DATETIME

def posa_interval(listi, date_from, date_to, duration):
        if not duration or date_to<=date_from: return list(listi) # nothing to add
        # location
        begin = bisect.bisect_left(listi, (date_from, 0) )
        duration2 = 0
        new = listi[:begin] # remains left
        if begin:
            duration2 = listi[begin-1][1] # update last duration
        if len(listi) == begin or listi[begin][0] != date_from:
            new += [(date_from, duration2 + duration)] # interval between last and actual
        while begin < len(listi) and listi[begin][0] < date_to: # adding value until date_to
            duration2 = listi[begin][1]  # update last duration
            if not new or new[-1][1] != duration2 + duration: # no same value to last interval
                new += [(listi[begin][0], duration2 + duration)]
            begin += 1
        if begin == len(listi) or listi[begin][0] != date_to: # interval no coincidence, some interval between actual and next
            new += [(date_to, duration2)]
        while begin < len(listi) and listi[begin][1] == duration2:
            begin += 1 # next values equals to last one
        new += listi[begin:] # remains right
        return new

class school_teacher(osv.osv):
    _name = 'school.teacher'
    _inherit = 'school.teacher'

    def _compute_assigned_hours(self, cr, uid, ids, field_name, arg, context=None):
        if not context: context = {}
        ret = dict.fromkeys(ids,{'max_assigned_hours': -1, 'min_assigned_hours': -1})
        date_from = max(context.get('date_from', _MIN_DATETIME) or _MIN_DATETIME, _MIN_DATETIME)
        date_to = min(context.get('date_to', _MAX_DATETIME) or _MAX_DATETIME, _MAX_DATETIME)
        cr.execute("""
        SELECT td.teacher_id,td.limit_from,td.limit_to,iwl.duration
        FROM school_teacher_data AS td
            INNER JOIN school_impartition_week_line AS iwl ON td.iwl_id=iwl.id
        WHERE td.teacher_id IN %(teacher_ids)s AND td.limit_from < %(date_to)s AND td.limit_to > %(date_from)s
        """, {'teacher_ids': tuple(ids), 'date_from': date_from, 'date_to': date_to} )

        dicci = {}
        for teacher_id,limit_from,limit_to,duration in cr.fetchall():
            if teacher_id not in dicci:
                dicci[teacher_id] = [(date_from,0)]
            dicci[teacher_id] = posa_interval(dicci[teacher_id], max(limit_from,date_from), min(limit_to,date_to), duration)

        for (teacher_id,llista) in dicci.items():
            ret[teacher_id]['max_assigned_hours'] = max(x[1] for x in llista)
            ret[teacher_id]['min_assigned_hours'] = min(x[1] for x in llista)
        return ret

    def _search_assigned_hours(self, cr, uid, obj, name, args, context = None):
        if not context: context = {}
        args2 = [(field,op,value) for (field,op,value) in args if field not in ('max_assigned_hours','min_assigned_hours')]
        if len(args) > len(args2):
            teacher_ids = self.search(cr, uid, args2, context = context)
            date_from = max(context.get('date_from', _MIN_DATETIME) or _MIN_DATETIME, _MIN_DATETIME)
            date_to = min(context.get('date_to', _MAX_DATETIME) or _MAX_DATETIME, _MAX_DATETIME)
            td_ids = self.pool.get('school.teacher_data').search(cr, uid, [('teacher_id','in',teacher_ids),('limit_to','>',date_from),('limit_from','<',date_to)])

            dicci = {}
            for item in self.pool.get('school.teacher_data').browse(cr, uid, td_ids):
                if item.teacher_id.id not in dicci:
                    dicci[item.teacher_id.id] = [(date_from,0)]
                dicci[item.teacher_id.id] = posa_interval(dicci[item.teacher_id.id], max(item.limit_from,date_from), min(item.limit_to,date_to), item.iwl_id.duration)

            teacher_ids = []
            for (teacher_id,llista) in dicci.items():
                minim = min(x[1] for x in llista)
                maxim = max(x[1] for x in llista)
                f = True
                for arg in args:
                    value = minim
                    if arg[0] == 'max_assigned_hours':
                        value = maxim
                    elif arg[0] == 'min_assigned_hours':
                        value = minim
                    else:
                        continue
                    if arg[1] == '=':
                        val = eval(value + '==' + arg[2], locals())
                    elif arg[1] in ['<', '>', 'in', 'not in', '<=', '>=', '<>']:
                        val = eval(value + arg[1] + arg[2], locals())
                    f = f and val
                if f:
                    teacher_ids.append(teacher_id)

            return [('id','in',teacher_ids)]
        return args2
        
    _columns = {
        'max_week_hours': fields.float('Max hours', help="Max. week hours to help assign "),
        'min_week_hours': fields.float('Min hours', help="Min. week hours to help assign "),
        'teacher_data_ids': fields.one2many('school.teacher_data', 'teacher_id', string='Teacher datas', ),
        'max_assigned_hours': fields.function(_compute_assigned_hours, fnct_search = _search_assigned_hours, type='float', method=True, string='Max assig hours', multi = 'assigned_hours',),
        'min_assigned_hours': fields.function(_compute_assigned_hours, fnct_search = _search_assigned_hours, type='float', method=True, string='Min assig hours', multi = 'assigned_hours',),
    }

    _sql_constraints = [
        ('max_greater_or_equal_than_min', 'CHECK (max_week_hours >= min_week_hours)', "Max hours should be greater or equal than min hours"),
        ('min_hours_not_negative', 'CHECK (min_week_hours >= 0)', "Max hours should be not negative"),
        ]

school_teacher()

class school_teacher_change_hours(osv.osv_memory):
    _name = 'school.teacher_change_hours'

    _columns = {
        'teacher_ids': fields.many2many('school.teacher', 'teacher_change_hours_wizard_rel', 'wizard_id', 'teacher_id', string='Teachers', ),
        'max_hours': fields.float('Max Hours', help='Max hours',),
        'min_hours': fields.float('Min Hours', help='Min hours',),
    }

    _defaults = {
        'teacher_ids': lambda self, cr, uid, context = {}: context.get('active_ids', []),
    }

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_change(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids):
            teacher_ids = [x.id for x in item.teacher_ids]
            self.pool.get('school.teacher').write(cr, uid, teacher_ids, {'max_week_hours': item.max_hours, 'min_week_hours': item.min_hours})
        return {'type': 'ir.actions.act_window_close'}

school_teacher_change_hours()

class school_teacher_course_suitability(osv.osv):
    _name = 'school.teacher_course_suitability'

    def _compute_assigned_hours(self, cr, uid, ids, field_name, arg, context=None):
        if not context: context = {}
        ret = dict.fromkeys(ids,{'max_assigned_hours': -1, 'min_assigned_hours': -1, 'total_max_assigned_hours': -1, 'total_min_assigned_hours': -1})
        date_from = max(context.get('date_from', _MIN_DATETIME) or _MIN_DATETIME, _MIN_DATETIME)
        date_to = min(context.get('date_to', _MAX_DATETIME) or _MAX_DATETIME, _MAX_DATETIME)
        for cs in self.browse(cr, uid, ids, context = context):
            td_ids = self.pool.get('school.teacher_data').search(cr, uid, [('teacher_id','=',cs.teacher_id.id),('iwl_id.classe_id.course_id','=',cs.course_id.id),('limit_to','>',date_from),('limit_from','<',date_to)])
            llista = [(date_from,0)]
            for td in self.pool.get('school.teacher_data').browse(cr, uid, td_ids):
                llista = posa_interval(llista, max(td.limit_from,date_from), min(td.limit_to,date_to), td.iwl_id.duration)
            ret[cs.id]['max_assigned_hours'] = max(x[1] for x in llista)
            ret[cs.id]['min_assigned_hours'] = min(x[1] for x in llista)
            ret[cs.id]['total_max_assigned_hours'] = cs.teacher_id.max_assigned_hours
            ret[cs.id]['total_min_assigned_hours'] = cs.teacher_id.min_assigned_hours
        return ret

    def _search_assigned_hours(self, cr, uid, obj, name, args, context = None):
        if not context: context = {}
        
        args2 = []
        teacher_args_transform = {'total_max_assigned_hours': 'max_assigned_hours', 'total_min_assigned_hours': 'min_assigned_hours'}
        teacher_args = [(teacher_args_transform[field],op,value) for (field,op,value) in args if field in teacher_args_transform]
        if teacher_args:
            for (field,op,value) in args:
                parts = field.split('.')
                if parts[0] == 'teacher_id':
                    if len(parts) < 2: parts.append('id')
                    teacher_args.append( ('.'.join(parts[1:]),op,value) )
                else:
                    args2 += (field,op,value)
            teacher_ids = self.pool.get('school.teacher').search(cr, uid, teacher_args, context = context)
            args2 += [('teacher_id','in',teacher_ids)]
        else:
            args2 = args

        args3 = [(field,op,value) for (field,op,value) in args2 if field not in ['max_assigned_hours','min_assigned_hours']]
        if len(args3) < len(args2):
            cs_ids = self.search(cr, uid, args3, context = context)
            cd_ids2 = []
            for cs in self.browse(cr, uid, cs_ids):
                date_from = max(context.get('date_from', _MIN_DATETIME) or _MIN_DATETIME, _MIN_DATETIME)
                date_to = min(context.get('date_to', _MAX_DATETIME) or _MAX_DATETIME, _MAX_DATETIME)
                td_ids = self.pool.get('school.teacher_data').search(cr, uid, [('teacher_id','=',cs.teacher_id.id),('iwl_id.classe_id.course_id','=',cs.course_id.id),('limit_to','>',date_from),('limit_from','<',date_to)])

                llista = [(date_from,0)]
                for item in self.pool.get('school.teacher_data').browse(cr, uid, td_ids):
                    llista = posa_interval(llista, max(item.limit_from,date_from), min(item.limit_to,date_to), item.iwl_id.duration)

                minim = min(x[1] for x in llista)
                maxim = max(x[1] for x in llista)
                f = True
                for arg in args:
                    value = arg[0]=='min_assigned_hours' and minim or arg[0]=='max_assigned_hours' and maxim or -1
                    if value < 0: continue
                    if arg[1] == '=':
                        val = eval(value + '==' + arg[2], locals())
                    elif arg[1] in ['<', '>', 'in', 'not in', '<=', '>=', '<>']:
                        val = eval(value + arg[1] + arg[2], locals())
                    f = f and val
                if f:
                    cd_ids2.append(cs.id)
            args3 += [('id','in',cs_ids2)]
        return args3
        
    _columns = {
        'teacher_id': fields.many2one('school.teacher', 'Teacher', ondelete='cascade', ),
        'course_id': fields.many2one('school.course', 'Course', ondelete='cascade', ),
        'percentage': fields.float('Percentage', help='Percentage suitability', ),
        'max_week_hours': fields.float('Max hours', help="Max. week hours to help assign "),
        'min_week_hours': fields.float('Min hours', help="Min. week hours to help assign "),
        'total_max_hours': fields.related('teacher_id', 'max_week_hours', type='float', string="Total max", ),
        'total_min_hours': fields.related('teacher_id', 'min_week_hours', type='float', string="Total min", ),
        'max_assigned_hours': fields.function(_compute_assigned_hours, fnct_search = _search_assigned_hours, type='float', method=True, string='Max assig hours', multi = 'assigned_hours',),
        'min_assigned_hours': fields.function(_compute_assigned_hours, fnct_search = _search_assigned_hours, type='float', method=True, string='Min assig hours', multi = 'assigned_hours',),
        'total_max_assigned_hours': fields.function(_compute_assigned_hours, fnct_search = _search_assigned_hours, type='float', method=True, string='Total Max assig hours', multi = 'assigned_hours',),
        'total_min_assigned_hours': fields.function(_compute_assigned_hours, fnct_search = _search_assigned_hours, type='float', method=True, string='Total Min assig hours', multi = 'assigned_hours',),
    }

    _sql_constraints = [
        ('teacher_course_unique', 'UNIQUE(teacher_id,course_id)', 'Only one register for the same teacher and course'),
        ('max_greater_or_equal_than_min', 'CHECK(max_week_hours >= min_week_hours)', "Max hours should be greater or equal than min hours"),
        ('min_hours_not_negative', 'CHECK(min_week_hours >= 0)', "Max hours should be not negative"),
        ('percentage_between_0_and_100', 'CHECK(percentage BETWEEN 0 AND 100)', "Percentage should be between 0 and 100"),
        ]

school_teacher_course_suitability()

class school_teacher_suitability_change(osv.osv_memory):
    _name = 'school.teacher_suitability_change'

    _columns = {
        'teacher_ids': fields.many2many('school.teacher', 'teacher_suitability_change_teacher_rel', 'wizard_id', 'teacher_id', string='Teachers', ),
        'course_ids': fields.many2many('school.course', 'teacher_suitability_change_course_rel', 'wizard_id', 'course_id', string='Courses', ),
        'percentage': fields.float('Percentage', ),
        'max_hours': fields.float('Max Hours', help='Max hours',),
        'min_hours': fields.float('Min Hours', help='Min hours',),
    }

    _defaults = {
        'teacher_ids': lambda self, cr, uid, context = {}: context.get('active_ids', []),
    }

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_change(self, cr, uid, ids, context=None):
        obj = self.pool.get('school.teacher_course_suitability')
        for item in self.browse(cr, uid, ids):
            dicci = {'max_week_hours': item.max_hours, 'min_week_hours': item.min_hours, 'percentage': item.percentage,}
            ids_to_write = []
            for teacher in item.teacher_ids:
                for course in item.course_ids:
                    tcs_ids = obj.search(cr, uid, [('teacher_id', '=', teacher.id), ('course_id', '=', course.id)])
                    if tcs_ids:
                        ids_to_write += tcs_ids
                    else:
                        dicci2 = dict(dicci)
                        dicci2.update({'teacher_id': teacher.id, 'course_id': course.id,})
                        obj.create(cr, uid, dicci2, context = context)
            if ids_to_write:
                obj.write(cr, uid, ids_to_write, dicci, context = context)
        return {'type': 'ir.actions.act_window_close'}

school_teacher_suitability_change()

class school_impartition_week_line(osv.osv):
    _name = 'school.impartition_week_line'
    _inherit = 'school.impartition_week_line'
    
    def _compute_needed(self, cr, uid, ids, field_name, arg, context=None):
        ret = {}
        for iwl in self.browse(cr, uid, ids):
            ret[iwl.id] = (iwl.teachers_needed <= len(iwl.teachers))
        return ret

    def _get_iwl_ids_from_td_ids(self, cr, uid, ids, context=None):
        ret = set()
        for td in self.browse(cr, uid, ids):
            ret.add(td.iwl_id.id)
        return list(ret)
    
    _columns = {
        'teachers_needed': fields.integer('Teachers needed', ),
        'teachers_recommended': fields.integer('Teachers recommended', ),
        'needed_completed': fields.function(_compute_needed, type='boolean', method=True, string='Teachers completed',
                                            store={'school.teacher_data': (_get_iwl_ids_from_td_ids, ['iwl_id', ], 10)},
                                            ),
    }
    
    _defaults = {
        'teachers_needed': lambda * a: 1,
        'teachers_recommended': lambda * a: 1,
    }
    
    _sql_constraints = [
        ('needed_and_recommended_check', 'CHECK (teachers_needed <= teachers_recommended)', "The number required should be smaller than the recommended"),
        ('needed_not_negative', 'CHECK (teachers_needed >= 0)', "The number required should be not negative"),
        ]

school_impartition_week_line()

class school_iwl_change_teachers_number_wizard(osv.osv_memory):
    _name = 'school.iwl_change_teachers_wizard'

    _columns = {
        'iwl_ids': fields.many2many('school.impartition_week_line', 'iwl_block_wizard_rel', 'wizard_id', 'iwl_id', string='IWLs', ),
        'teachers_needed': fields.integer('Teachers needed', ),
        'teachers_recommended': fields.integer('Teachers recommended', ),

    }

    _defaults = {
        'iwl_ids': lambda self, cr, uid, context = {}: [(6,0,context.get('active_ids', []))],
        'teachers_needed': lambda * a: 1,
        'teachers_recommended': lambda * a: 1,
    }

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_block(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids):
            iwl_ids = [x.id for x in item.iwl_ids]
            self.pool.get('school.impartition_week_line').write(cr, uid, iwl_ids, {'teachers_needed': item.teachers_needed, 'teachers_recommended': item.teachers_recommended})

school_iwl_change_teachers_number_wizard()



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
